package cz.sa.shiftPlanner.dao;

import java.util.List;

import org.joda.time.DateTime;

import cz.sa.shiftPlanner.entities.BusConnectionVO;
import cz.sa.shiftPlanner.entities.LineGroupVO;
import cz.sa.shiftPlanner.entities.StationVO;
import cz.sa.shiftPlanner.utils.TimeRange;

public interface BusConnectionDAO {
	/**
     * gets Connection by its id
     * @param id
     * @throws IllegalArgumentException when argument null
     * @return Connection or NULL when not found
     */
	BusConnectionVO get(Long id);
    
    /**
     * Persists connection
     * @throws ArgumentException when connection is already persisted
     * @throws IllegalArgumentException when argument null
     * @param connection
     */
    void create(BusConnectionVO connection);

    /**
     * Updates connection
     * @throws ArgumentException when connection is not persisted
     * @throws IllegalArgumentException when argument null
     * @param connection
     */
    void update(BusConnectionVO connection);

    /**
     * Removes persisted connection
     * @throws ArgumentException when connection is not persisted
     * @throws IllegalArgumentException when argument null
     * @param connection 
     */
    void remove(BusConnectionVO connection);

    /**
     * Retrieves all connections from source
     * @return 
     */
    List<BusConnectionVO> findAll();

    List<BusConnectionVO> findConnectionByTime(DateTime departure, DateTime arrival);
    
    BusConnectionVO findConnectionByConcreteFigure(DateTime departure, DateTime arrival, StationVO departureCity, StationVO arrivalCity, LineGroupVO line);
    
    /**
     * Method expect begin of day and end of next day in parameter range
     * @param range
     * @param lg
     * @return
     */
    List<BusConnectionVO> findConnectionByTimeRange(TimeRange range, LineGroupVO lg);

	void save(BusConnectionVO vo);

	List<BusConnectionVO> findSupportsByTimeRange(DateTime day, Long groupLine);
	
	BusConnectionVO findFirstEmpty(LineGroupVO lg);
}
