package cz.sa.shiftPlanner.dao;

import java.util.List;

import org.joda.time.DateTime;

import cz.sa.shiftPlanner.dto.BusTimetableDTO;
import cz.sa.shiftPlanner.entities.BusTimetableVO;
import cz.sa.shiftPlanner.enums.WeekDay;


public interface BusTimetableDAO {

	/**
     * gets timetable by its id
     * @param id
     * @throws IllegalArgumentException when argument null
     * @return timetable or NULL when not found
     */
	BusTimetableVO get(Long id);
    
    /**
     * Persists timetable
     * @throws ArgumentException when timetable is already persisted
     * @throws IllegalArgumentException when argument null
     * @param timetable
     */
    void create(BusTimetableVO timetable);

    /**
     * Updates timetable
     * @throws ArgumentException when timetable is not persisted
     * @throws IllegalArgumentException when argument null
     * @param timetable 
     */
    void update(BusTimetableVO timetable);

    /**
     * Removes persisted timetable
     * @throws ArgumentException when timetable is not persisted
     * @throws IllegalArgumentException when argument null
     * @param timetable
     */
    void remove(BusTimetableVO timetable);
    
    List<BusTimetableVO> findAll();
    
    @SuppressWarnings("unchecked")
    List<BusTimetableVO> findByValidity(DateTime validity);
    
    List<BusTimetableVO> findByLineGroupId(Long id, DateTime validity);
    
    List<BusTimetableVO> findByDepCityId(Long id, DateTime validity);
    
    List<BusTimetableVO> findByLineGroupIdAndDay(Long id, WeekDay day, DateTime validity);
    
    List<BusTimetableVO> findByLineGroupIdAndDay(Long id, DateTime day, DateTime validity);
    
    BusTimetableVO findByAllStats(Long lineGroupId, Long departureCityId, Long arrivalCityID, DateTime departureTime, DateTime arrivalTime, int weekDay, DateTime validity);

	void save(BusTimetableVO vo);
    
}
