package cz.sa.shiftPlanner.dao;

import java.util.List;

import org.joda.time.DateTime;

import cz.sa.shiftPlanner.entities.BusTimetableVO;
import cz.sa.shiftPlanner.entities.BusTurnsTimetableVO;
import cz.sa.shiftPlanner.enums.WeekDay;


public interface BusTurnsTimetableDAO {

	/**
     * gets timetable by its id
     * @param id
     * @throws IllegalArgumentException when argument null
     * @return timetable or NULL when not found
     */
	BusTurnsTimetableVO get(Long id);
    
    /**
     * Persists timetable
     * @throws ArgumentException when timetable is already persisted
     * @throws IllegalArgumentException when argument null
     * @param timetable
     */
    void create(BusTurnsTimetableVO timetable);

    /**
     * Updates timetable
     * @throws ArgumentException when timetable is not persisted
     * @throws IllegalArgumentException when argument null
     * @param timetable 
     */
    void update(BusTurnsTimetableVO timetable);

    /**
     * Removes persisted timetable
     * @throws ArgumentException when timetable is not persisted
     * @throws IllegalArgumentException when argument null
     * @param timetable
     */
    void remove(BusTurnsTimetableVO timetable);
  
    
    List<BusTurnsTimetableVO> findAll();
    
    List<BusTurnsTimetableVO> findByLineGroupIdAndDay(Long id, WeekDay day, Integer week, DateTime validity);
    
    BusTurnsTimetableVO findByDayAndTimetable(BusTimetableVO timetable, WeekDay day, Integer week);
    
    BusTurnsTimetableVO findByDayAndTimetable(BusTimetableVO timetable, WeekDay day, DateTime validity, Integer week);
    
    BusTurnsTimetableVO findByDayAndTimetableOB(BusTimetableVO timetable, WeekDay day);

	  List<BusTurnsTimetableVO> findByTimetable(Long timId);
}
