package cz.sa.shiftPlanner.dao;

import java.util.List;

import org.joda.time.DateTime;

import cz.sa.shiftPlanner.dto.CityDTO;
import cz.sa.shiftPlanner.entities.CityVO;
import cz.sa.shiftPlanner.entities.PersonVO;
import cz.sa.shiftPlanner.entities.StationVO;

public interface PersonDAO {

    /**
     * gets Person by its id
     * @param id
     * @throws IllegalArgumentException when argument null
     * @return Person or NULL when not found
     */
    PersonVO get(Long id);
    
    /**
     * Persists person
     * @throws ArgumentException when person is already persisted
     * @throws IllegalArgumentException when argument null
     * @param person
     */
    void create(PersonVO person);

    /**
     * Updates person
     * @throws ArgumentException when person is not persisted
     * @throws IllegalArgumentException when argument null
     * @param person 
     */
    void update(PersonVO person);

    /**
     * Removes persisted person
     * @throws ArgumentException when person is not persisted
     * @throws IllegalArgumentException when argument null
     * @param person 
     */
    void remove(PersonVO person);

    /**
     * Retrieves all persons from source
     * @return 
     */
    List<PersonVO> findAll();

    /**
     * Retrieve person by its Personal ID
     * @param personalId
     * @return 
     */
   
    
    List<PersonVO> findByName(String name, String surname);
    
    PersonVO findByParameters(String name, String surname, String mail, String phone);
    
    List<PersonVO> findByLineGroup(Long id);
    PersonVO findByPersonalNumber(Long personalNumber);

	List<PersonVO> findByStations(List<StationVO> stations);

	List<PersonVO> findAllByTrialDay(DateTime day);

  List<PersonVO> findByDepCity(CityVO city);

  List<PersonVO> findByHPP(String jobTime);
    
}
