package cz.sa.shiftPlanner.dao;

import java.util.List;

import org.joda.time.DateTime;

import cz.sa.shiftPlanner.entities.BusTurnsTimetableVO;
import cz.sa.shiftPlanner.entities.RuleVO;
import cz.sa.shiftPlanner.enums.WeekDay;

public interface RuleDAO {

	/**
     * gets rule by its id
     * @param id
     * @throws IllegalArgumentException when argument null
     * @return rule or NULL when not found
     */
    RuleVO get(Long id);
    
    /**
     * Persists rule
     * @throws ArgumentException when rule is already persisted
     * @throws IllegalArgumentException when argument null
     * @param rule
     */
    void create(RuleVO rule);

    /**
     * Updates rule
     * @throws ArgumentException when rule is not persisted
     * @throws IllegalArgumentException when argument null
     * @param rule 
     */
    void update(RuleVO rule);

    /**
     * Removes persisted rule
     * @throws ArgumentException when rule is not persisted
     * @throws IllegalArgumentException when argument null
     * @param rule
     */
    void remove(RuleVO rule);
    
    /**
     * Removes persisted rule
     * @throws ArgumentException when rule is not persisted
     * @throws IllegalArgumentException when argument null
     * @param rule
     */
    List<RuleVO> findByWeekAndDay(WeekDay day, Integer week, DateTime validity);
    
    RuleVO findByWeekAndTurn(BusTurnsTimetableVO turn, Integer week);

    List<RuleVO> findByTurn(BusTurnsTimetableVO t);
}
