package cz.sa.shiftPlanner.daoImpl;

import java.util.Arrays;
import java.util.List;

import org.joda.time.DateTime;
import org.springframework.stereotype.Repository;

import cz.sa.shiftPlanner.dao.BusTimetableDAO;
import cz.sa.shiftPlanner.entities.BusTimetableVO;
import cz.sa.shiftPlanner.entities.BusTurnsTimetableVO;
import cz.sa.shiftPlanner.entities.RuleVO;
import cz.sa.shiftPlanner.entities.SymbolVO;
import cz.sa.shiftPlanner.enums.WeekDay;
import cz.sa.shiftPlanner.infrastructure.dao.AbstractDao;

@Repository
public class BusTimetableDAOImpl extends AbstractDao implements BusTimetableDAO {

	@Override
	public BusTimetableVO get(Long id) {
		if(id==null) throw new IllegalArgumentException("timetable");
		return (BusTimetableVO) getSession().get(BusTimetableVO.class, id);
	}

	@Override
	public void create(BusTimetableVO timetable) {
		if ( timetable == null)
            throw new IllegalArgumentException("timetable");
        if (timetable.getId() != null)
            throw new IllegalArgumentException("Already persisted");

        getSession().save(timetable);
	}
	
	@Override
	public void save(BusTimetableVO timetable) {
		getSession().save(timetable);
	}

	@Override
	public void update(BusTimetableVO timetable) {
		if ( timetable == null)
            throw new IllegalArgumentException("timetable");
        if (timetable.getId() == null)
            throw new IllegalArgumentException("Already persisted");
        
        getSession().merge(timetable);
	}

	@Override
	public void remove(BusTimetableVO timetable) {
		if ( timetable == null)
            throw new IllegalArgumentException("timetable");
        if (timetable.getId() == null)
            throw new IllegalArgumentException("Already persisted");

        getSession().delete(timetable);
	}

	@Override
	public List<BusTimetableVO> findAll() {
		return getSession().createQuery("from BusTimetableVO").list();
	}

	@Override
	public List<BusTimetableVO> findByLineGroupId(Long id, DateTime validity) {
		if(id==null) throw new IllegalArgumentException("timetable");
		return getSession().createQuery("SELECT t from BusTimetableVO AS t left join t.symbol as symbols where" +
				" t.line.id = :line" +
				" AND (t.validity_from < :validity AND t.validity_to > :validity)" +
				" order by t.departureStation , t.departureTime")
				.setParameter("validity", validity)
				.setLong("line", id).list();
	}

/*
 * slouzi pro nacitani tocek pri selektu obecneho dne
 */
	@Override
	public List<BusTimetableVO> findByLineGroupIdAndDay(Long id, WeekDay day, DateTime validity) {
		List<WeekDay> days = Arrays.asList(WeekDay.values());
		switch(day){
			case MONDAY :
			case TUESDAY :
			case WEDNESDAY :
			case THURSDAY : 
			case FRIDAY :  
				return (List<BusTimetableVO>) getSession().createQuery(
					"SELECT time from BusTimetableVO AS time" +
					" left join time.symbol as symbols " +
					"left join fetch time.turns" +
					" where time.line.id = :line" +
					" AND (symbols.mark in ('x') or symbols.mark = :day  or symbols is null )" +
					" AND (time.validity_from < :validity AND time.validity_to > :validity)" +
					" order by time.departureStation , time.departureTime")
					.setParameter("day", String.valueOf(days.indexOf(day)+1))
					.setParameter("validity", validity)
					.setParameter("line", id)
					.list();
			case SATURDAY : 
				return (List<BusTimetableVO>) getSession().createQuery(
						"SELECT time from BusTimetableVO AS time" +
						" left join time.symbol as symbols " +
						"left join fetch time.turns" +
						" where time.line.id = :line" +
						" AND (symbols.mark in ('6') or symbols is null)" +
						" AND (time.validity_from < :validity AND time.validity_to > :validity)" +
						" order by time.departureStation , time.departureTime")
						.setParameter("validity", validity)
						.setParameter("line", id)
						.list();
			case SUNDAY :  
				return getSession().createQuery(
						"SELECT time from BusTimetableVO AS time" +
						" left join time.symbol as symbols " +
						"left join fetch time.turns" +
						" where time.line.id = :line" +
						" AND (symbols.mark in ('7', '+') or symbols is null)" +
						" AND (time.validity_from < :validity AND time.validity_to > :validity)" +
						" order by time.departureStation , time.departureTime")
						.setParameter("validity", validity)
						.setParameter("line", id)
						.list();
			case WORKDAY:
			case WHOLEWEEK:
				return getSession().createQuery(
						"SELECT time from BusTimetableVO AS time" +
								" left join time.symbol as symbols " +
								"left join fetch time.turns" +
								" where time.line.id = :line" +
								" AND (symbols.mark in ('1', 'x') or symbols is null)" +
								" AND (time.validity_from < :validity AND time.validity_to > :validity)" +
						" order by time.departureStation , time.departureTime")
						.setParameter("validity", validity)
						.setParameter("line", id)
						.list();
			default: throw new IllegalArgumentException("bad day");
		}
	}

	@Override
	public BusTimetableVO findByAllStats(Long lineGroupId,
			Long departureCityId, Long arrivalCityId, DateTime departureTime,
			DateTime arrivalTime, int weekDay, DateTime validity) {

		return (BusTimetableVO)getSession().createQuery(
				"SELECT time from BusTimetableVO AS time" +
						" left join time.symbol as symbols " +
						" where time.line.id = :line" +
						" AND time.departureCity.id = :depC" +
						" AND time.arrivalCity.id = :arrC" +
						" AND time.departureTime = :departure" +
						" AND time.arrivalTime = :arrival" +
						" AND (symbols.mark in (:mark, 'x') or symbols is null)" +
						" AND (time.validity_from < :validity AND time.validity_to > :validity)")
				.setParameter("validity", validity)
				.setParameter("departure", departureTime)
				.setParameter("arrival", arrivalTime)
				.setParameter("depC", departureCityId)
				.setParameter("arrC", arrivalCityId)
				.setParameter("line", lineGroupId)
				.setParameter("mark", String.valueOf(weekDay))
				.uniqueResult();
	}

	/*
	 * slouzi pro nacitani "vyjimecnych" dnu
	 */
	@Override
	public List<BusTimetableVO> findByLineGroupIdAndDay(Long id, DateTime day,
			DateTime validity) {

		int date = day.getDayOfWeek();
		if(date >= 1 && date <= 5)
			return getSession().createQuery(
					"SELECT time from BusTimetableVO AS time" +
					" left join time.symbol as symbols" +
					" left join fetch time.turns" +
					" left join time.timeCode as timecode" +
					" left join timecode.items as item" +
					" where time.line.id = :line" +
					" AND (symbols.mark in ('x') or symbols.mark = :day  or symbols is null )" +
					" AND (time.validity_from < :validity AND time.validity_to > :validity)" +
					" order by time.departureStation , time.departureTime")
					.setParameter("day", String.valueOf(date))
					.setParameter("validity", validity)
					.setParameter("line", id)
					.list();
		if(date == 6)
			return getSession().createQuery(
					"SELECT time from BusTimetableVO AS time" +
					" left join time.symbol as symbols" +
					" left join fetch time.turns" +
					" left join time.timeCode as timecode" +
					" left join timecode.items as item" +
					" where time.line.id = :line" +
					" AND (symbols.mark in ('6') or symbols is null)" +
					" AND (time.validity_from < :validity AND time.validity_to > :validity)" +
					" order by time.departureStation , time.departureTime")
					.setParameter("validity", validity)
					.setParameter("line", id)
					.list();
		if(date == 7)
			return getSession().createQuery(
					"SELECT time from BusTimetableVO AS time" +
					" left join time.symbol as symbols" +
					" left join fetch time.turns" +
					" left join time.timeCode as timecode" +
					" left join timecode.items as item" +
					" where time.line.id = :line" +
					" AND (symbols.mark in ('7', '+') or symbols is null)" +
					" AND (time.validity_from < :validity AND time.validity_to > :validity)" +
					//" AND item.type like ('INCLUDING') AND (item.from_ <= :date AND item.to_ >= :date)" +
					" order by time.departureStation , time.departureTime")
					//.setParameter("date", day)
					.setParameter("validity", validity)
					.setParameter("line", id)
					.list();
		return null;
	}

	@Override
	public List<BusTimetableVO> findByValidity(DateTime validity) {
		
		return getSession().createQuery("SELECT t from BusTimetableVO AS t  where" +
				" t.validity_from < :validity AND t.validity_to > :validity")
				.setParameter("validity", validity).list();
	}

	@Override
	public List<BusTimetableVO> findByDepCityId(Long id, DateTime validity) {
		if(id==null) throw new IllegalArgumentException("timetable");
		return getSession().createQuery("SELECT t from BusTimetableVO AS t left join t.symbol as symbols where" +
				" t.departureStation.city.id = :station" +
				" AND (t.validity_from < :validity AND t.validity_to > :validity)" +
				" order by t.line , t.departureTime")
				.setParameter("validity", validity)
				.setParameter("station", id).list();
	}


}
