package cz.sa.shiftPlanner.daoImpl;

import java.util.List;

import org.apache.xmlbeans.impl.xb.xsdschema.RestrictionDocument.Restriction;
import org.hibernate.Criteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;

import cz.sa.shiftPlanner.dao.NotificationDAO;
import cz.sa.shiftPlanner.dto.NotificationLoadDTO;
import cz.sa.shiftPlanner.entities.NotificationVO;
import cz.sa.shiftPlanner.enums.NotifType;
import cz.sa.shiftPlanner.infrastructure.dao.AbstractDao;

@Repository
public class NotificationDAOImpl extends AbstractDao implements NotificationDAO {

  @Override
  public void create(NotificationVO note) {

    if (note == null)
      throw new IllegalArgumentException("note");
    if (note.getId() != null)
      throw new IllegalArgumentException("Already persisted");

    getSession().save(note);
  }

  @Override
  public void update(NotificationVO note) {

    if (note == null)
      throw new IllegalArgumentException("note");
    if (note.getId() == null)
      throw new IllegalArgumentException("Already persisted");

    getSession().update(note);
  }

  @Override
  public void remove(NotificationVO note) {

    if (note == null)
      throw new IllegalArgumentException("note");
    if (note.getId() == null)
      throw new IllegalArgumentException("Already persisted");
    getSession().delete(note);
  }

  @Override
  public List<NotificationVO> findSelectedBy(NotificationLoadDTO load) {

    Criteria crit = getSession().createCriteria(NotificationVO.class).add(Restrictions.eq("notifType", load.getType()));
    if (load.isFindAll()) {
      crit.addOrder(Order.asc("isReaded"));
    }
    else {
      crit.add(Restrictions.eq("isReaded", false));
    }
    if (load.isAscending()) {
      crit.addOrder(Order.asc(load.getSortProperty()));
    }
    else {
      crit.addOrder(Order.desc(load.getSortProperty()));
    }
    return crit.list();
  }

  @Override
  public boolean checkUnreaded() {

    Long count = (Long) getSession().createQuery("select count(n) from NotificationVO as n where n.isReaded is false")
        .uniqueResult();
    if (count != 0)
      return true;
    else
      return true;
  }

}
