package cz.sa.shiftPlanner.entities;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import org.hibernate.annotations.Type;
import org.joda.time.DateTime;

import cz.sa.shiftPlanner.entities.infrastructure.AbstractVO;
import cz.sa.shiftPlanner.entities.infrastructure.UserVO;


@Entity
@Table(name = "person")
public class PersonVO extends AbstractVO{


	@Column(name="personalnumber", unique=true)
    private Long personalNumber;

	@Column(columnDefinition="varchar(50)", nullable=false)
	private String name;
	
	@Column(columnDefinition="varchar(50)", nullable=false)
	private String surname;
	
	@Column(columnDefinition="varchar(50)", nullable=true)
	private String email;
	
	@Column(columnDefinition="varchar(50)", nullable=true)
	private String phonenumber;
	
	@Column(nullable=false)
	private Boolean hpp;
	
	@Column(columnDefinition="varchar(20)", nullable=true)
	private String classification;
	
	@Column(nullable=true)
	private int weekWorkTime;

	@ManyToMany(fetch=FetchType.LAZY, cascade = {CascadeType.PERSIST, CascadeType.MERGE}, targetEntity = CityVO.class)
	@JoinTable(name = "person_city", 
	                 joinColumns = {@JoinColumn(name = "person_id") }, 
	          inverseJoinColumns = {@JoinColumn(name = "city_id") })
	private Collection<CityVO> city = new ArrayList<CityVO>();
	
	@ManyToMany(fetch=FetchType.LAZY, cascade = {CascadeType.PERSIST, CascadeType.MERGE}, targetEntity = LineVO.class)
  @JoinTable(name = "person_line", 
                   joinColumns = {@JoinColumn(name = "person_id") }, 
            inverseJoinColumns = {@JoinColumn(name = "line_id") })
	private Collection<LineVO> line = new ArrayList<LineVO>();

	@Type(type = "org.jadira.usertype.dateandtime.joda.PersistentDateTime")
	@Column(nullable = true)
	private DateTime dateUpdated;
	
	@OneToMany( mappedBy = "person", cascade = CascadeType.ALL)
	private List<UserVO> user;
	
	@OneToMany( mappedBy = "person", cascade = CascadeType.ALL)
	private List<TimeTrialVO> timeTrials;
	
	
	public List<TimeTrialVO> getTimeTrials() {
		return timeTrials;
	}

	public void setTimeTrials(List<TimeTrialVO> timeTrials) {
		this.timeTrials = timeTrials;
	}

	public List<UserVO> getUser() {
		return user;
	}

	public void setUser(List<UserVO> user) {
		this.user = user;
	}

	public DateTime getDateUpdated() {
		return dateUpdated;
	}

	public void setDateUpdated(DateTime dateUpdated) {
		this.dateUpdated = dateUpdated;
	}

	public String getPhonenumber() {
		return phonenumber;
	}

	public void setPhonenumber(String phonenumber) {
		this.phonenumber = phonenumber;
	}

	public Boolean getHpp() {
		return hpp;
	}

	public void setHpp(Boolean hpp) {
		this.hpp = hpp;
	}

	public String getClassification() {
		return classification;
	}

	public void setClassification(String classification) {
		this.classification = classification;
	}

	public int getWeekWorkTime() {
		return weekWorkTime;
	}

	public void setWeekWorkTime(int weekWorkTime) {
		this.weekWorkTime = weekWorkTime;
	}

	public Long getPersonalNumber() {
		return personalNumber;
	}

	public void setPersonalNumber(Long personalNumber) {
		this.personalNumber = personalNumber;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getSurname() {
		return surname;
	}

	public void setSurname(String surname) {
		this.surname = surname;
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

  public List<CityVO> getCity() {
    return new ArrayList(city);
  }

  public void setCity(List<CityVO> city) {
    this.city = city;
  }

  public List<LineVO> getLine() {
    return new ArrayList(line);
  }

  public void setLine(List<LineVO> line) {
    this.line = line;
  }

  @Override
	public String toString() {
		return name + "," + surname;
	}

	
}
