package cz.sa.shiftPlanner.servicesImpl;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.dozer.Mapper;
import org.joda.time.DateTime;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import cz.sa.shiftPlanner.dao.BusTimetableDAO;
import cz.sa.shiftPlanner.dao.BusTurnsTimetableDAO;
import cz.sa.shiftPlanner.dto.BusConnectionDTO;
import cz.sa.shiftPlanner.dto.BusTimetableDTO;
import cz.sa.shiftPlanner.dto.BusTurnsTimetableDTO;
import cz.sa.shiftPlanner.dto.CityDTO;
import cz.sa.shiftPlanner.entities.BusTimetableVO;
import cz.sa.shiftPlanner.entities.BusTurnsTimetableVO;
import cz.sa.shiftPlanner.enums.WeekDay;
import cz.sa.shiftPlanner.services.BusTurnsTimetableService;
import cz.sa.shiftPlanner.utils.BusTurnsTimetableListComparator;
import cz.sa.shiftPlanner.utils.DateFormatter;
import cz.sa.shiftPlanner.utils.Tuple;

@Service
@Transactional(propagation = Propagation.REQUIRED)
public class BusTurnsTimetableServiceImpl implements BusTurnsTimetableService {

	private BusTimetableDAO busTimetableDAO;
	private BusTurnsTimetableDAO busTurnsTimetableDAO;
	
	private Mapper mapper; 
	
	public Mapper getMapper() {
		return mapper;
	}

	public void setMapper(Mapper mapper) {
		this.mapper = mapper;
	}

	public BusTimetableDAO getBusTimetableDAO() {
		return busTimetableDAO;
	}

	public void setBusTimetableDAO(BusTimetableDAO busTimetableDAO) {
		this.busTimetableDAO = busTimetableDAO;
	}

	public BusTurnsTimetableDAO getBusTurnsTimetableDAO() {
		return busTurnsTimetableDAO;
	}

	public void setBusTurnsTimetableDAO(BusTurnsTimetableDAO busTurnsTimetableDAO) {
		this.busTurnsTimetableDAO = busTurnsTimetableDAO;
	}

	
	@Override
	@Transactional(propagation = Propagation.REQUIRED)
	public void create(BusTurnsTimetableDTO timetable) {

		if (timetable == null)
			throw new IllegalArgumentException("timetable is null");
		BusTurnsTimetableVO b = mapper.map(timetable, BusTurnsTimetableVO.class);

		busTurnsTimetableDAO.create(b);
		mapper.map( b, timetable);
	}

	/**
	 * Dela save or update
	 */
	@Override
	@Transactional(propagation = Propagation.REQUIRED)
	public void update(BusTurnsTimetableDTO timetable) {

		if (timetable == null)
			throw new IllegalArgumentException("turn is null");

		BusTurnsTimetableVO vo = mapper.map(timetable, BusTurnsTimetableVO.class);
		busTurnsTimetableDAO.update(vo);
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED)
	public void remove(BusTurnsTimetableDTO timetable) {
		busTurnsTimetableDAO.remove(busTurnsTimetableDAO.get(timetable.getId()));
	}

	
	
	@Override
	@Transactional(propagation = Propagation.REQUIRED, readOnly = true)
	public Map<Tuple<CityDTO, CityDTO>, List<BusTurnsTimetableDTO>> findByLineGroupId(Long id) {
		
		return null;
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED, readOnly = true)
	public List<List<BusTurnsTimetableDTO>> findByLineGroupIdAndDay(Long id, WeekDay day, Integer week, DateTime validity) {
		
		List<BusTurnsTimetableDTO> list = new ArrayList<BusTurnsTimetableDTO>();
		for(BusTurnsTimetableVO p : busTurnsTimetableDAO .findByLineGroupIdAndDay(id, day, week, validity)){
			list.add(mapper.map(p, BusTurnsTimetableDTO.class));
		}
		List<List<BusTurnsTimetableDTO>> sortedList = sortListToLists(list);
		Collections.sort(sortedList, new BusTurnsTimetableListComparator());
		return sortedList;
	}

	private List<List<BusTurnsTimetableDTO>> sortListToLists(List<BusTurnsTimetableDTO> list) {

		List<List<BusTurnsTimetableDTO>> result = new ArrayList<List<BusTurnsTimetableDTO>>();
		if(list.size()==0) return result;
		
		int coupleNumberLoc = list.get(0).getCoupleNumber();
			
		List<BusTurnsTimetableDTO> listLoc = new ArrayList<BusTurnsTimetableDTO>();
		
		for(BusTurnsTimetableDTO dto : list){
			if(dto.getCoupleNumber() == null) continue;
			if(dto.getCoupleNumber() == coupleNumberLoc){
				if(!listLoc.contains(dto))
					listLoc.add(dto);
			}else{
				result.add(listLoc);
				coupleNumberLoc = dto.getCoupleNumber();
				listLoc = new ArrayList<BusTurnsTimetableDTO>();
				listLoc.add(dto);
			}
		}
		if(!result.contains(listLoc)) result.add(listLoc);
		return result;
	}

	@Override
	public void removeList(List<BusTurnsTimetableDTO> turns, boolean manageBohtWeeks) {

		List<WeekDay> days = Arrays.asList(WeekDay.values());
		for(BusTurnsTimetableDTO t : turns){
			
			if(t.getWeekDay().equals(WeekDay.WHOLEWEEK)){
				removeTurns(days, t, 7, manageBohtWeeks);
			}else if(t.getWeekDay().equals(WeekDay.WORKDAY)){
				removeTurns(days, t, 5, manageBohtWeeks);
			}else{
				removeTurn(t, manageBohtWeeks);
			}
		}
	}
	
	private void removeTurn(BusTurnsTimetableDTO t, boolean manageBohtWeeks){
		if(manageBohtWeeks){
			BusTimetableVO tim = mapper.map(t.getTimetable(),BusTimetableVO.class);
			BusTurnsTimetableVO newT = busTurnsTimetableDAO.findByDayAndTimetable(tim, t.getWeekDay(), 1);
			if(newT != null){
				busTurnsTimetableDAO.remove(newT);
			}
			newT = busTurnsTimetableDAO.findByDayAndTimetable(tim, t.getWeekDay(), 2);
			if(newT != null){
				busTurnsTimetableDAO.remove(newT);
			}
		}else{
			remove(t);
		}
	}
	
	private void removeTurns(List<WeekDay> days, BusTurnsTimetableDTO t, int numberOfDays, boolean manageBohtWeeks) {

		Logger logger = Logger.getLogger(BusTurnsTimetableServiceImpl.class);
		for(int i=0 ; i<numberOfDays; i++){
			BusTurnsTimetableVO newT;
			BusTimetableVO tim = null;
			/*//TODO change validity in method calling and set validity in new turn...kvuli platnosti jizdnich radu...otazka jestli je to potreba
			try{
				 contr= busTimetableDAO.findByAllStats(
						t.getTimetable().getLine().getId(),
						t.getTimetable().getDepartureCity().getId(),
						t.getTimetable().getArrivalCity().getId(),
						t.getTimetable().getDepartureTime(),
						t.getTimetable().getArrivalTime(),
						i+1, new DateTime());
			}catch(IncorrectResultSizeDataAccessException ex){
				logger.log(Level.WARN, "found duplicite record in wholeweek");
			}
			if(contr != null){
				if(t.getId()!= null){
					 newT = busTurnsTimetableDAO.findByDayAndTimetable(contr, days.get(i));
					if(newT == null) continue;
					busTurnsTimetableDAO.remove(newT);
					contr.getTurns().remove(newT);
				}
			}
			*/
			tim = mapper.map(t.getTimetable(),BusTimetableVO.class);
			if(manageBohtWeeks){
				newT = busTurnsTimetableDAO.findByDayAndTimetable(tim, days.get(i), 1);
				if(newT != null){
					busTurnsTimetableDAO.remove(newT);
				}
				newT = busTurnsTimetableDAO.findByDayAndTimetable(tim, days.get(i), 2);
				if(newT != null){
					busTurnsTimetableDAO.remove(newT);
				}
			}else{
				newT = busTurnsTimetableDAO.findByDayAndTimetable(tim, days.get(i), t.getWeek());
				if(newT != null){
					busTurnsTimetableDAO.remove(newT);
				}
			}
		}
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED)
	public void saveList(List<BusTurnsTimetableDTO> turns, DateTime validity, boolean manageBohtWeeks) {
		
		List<WeekDay> days = Arrays.asList(WeekDay.values());
		for(BusTurnsTimetableDTO t : turns){

			if(t.getWeekDay().equals(WeekDay.WHOLEWEEK)){
				generateTurns(days, t, 7, manageBohtWeeks);
			}else if(t.getWeekDay().equals(WeekDay.WORKDAY)){
				generateTurns(days, t, 5, manageBohtWeeks);
			}else{
				//pokud nastavuju novy vyjimecny den pak musim zaznamy z klasickeho dne prenastavit na vyjimky (musi byt zachovany puvodni zaznamy)
				if(validity != null && t.getValidity_from() == null){
					t.setValidity_from(validity);
					t.setValidity_to(validity);
					t.setId(null);
					create(t);
				}else{
					//save or update
					generateTurn(t, manageBohtWeeks);
				}
			}
		}
	}

	private void generateTurn( BusTurnsTimetableDTO t, boolean manageBohtWeeks) {
		//normalniho zaznamu se muze tykat sudy a lichy tyden a kdzy je zaskrtnute policko pro vypleni obou, tak zkopiruju zaznamy
		if(manageBohtWeeks){
			BusTimetableVO tim = mapper.map(t.getTimetable(),BusTimetableVO.class);
			//lichy tyden
			BusTurnsTimetableVO newT = busTurnsTimetableDAO.findByDayAndTimetable(tim, t.getWeekDay(), 1);
			if(newT == null){
				newT = new BusTurnsTimetableVO();
				newT.setWeekDay(t.getWeekDay());
				newT.setTimetable(tim);
			}
			newT.setCoupleNumber(t.getCoupleNumber());
			newT.setWeek(1);
			busTurnsTimetableDAO.update(newT);
			
			//sudy tyden
			newT = busTurnsTimetableDAO.findByDayAndTimetable(tim, t.getWeekDay(), 2);
			if(newT == null){
				newT = new BusTurnsTimetableVO();
				newT.setWeekDay(t.getWeekDay());
				newT.setTimetable(tim);
			}
			newT.setCoupleNumber(t.getCoupleNumber());
			newT.setWeek(2);
			busTurnsTimetableDAO.update(newT);
		//jinak updatuju nebo vytvorim jeden
		}else{
			update(t);
		}
	}

	private void generateTurns(List<WeekDay> days, BusTurnsTimetableDTO t, int numberOfDays, boolean manageBohtWeeks) {
		Logger logger = Logger.getLogger(BusTurnsTimetableServiceImpl.class);
		for(int i=0 ; i<numberOfDays; i++){
			BusTurnsTimetableVO newT;
			BusTimetableVO tim = null;
			/*//TODO change validity in method calling and set validity in new turn...kvuli platnosti jizdnich radu...otazka jestli je to potreba
			try{
				 contr= busTimetableDAO.findByAllStats(
						t.getTimetable().getLine().getId(),
						t.getTimetable().getDepartureCity().getId(),
						t.getTimetable().getArrivalCity().getId(),
						t.getTimetable().getDepartureTime(),
						t.getTimetable().getArrivalTime(),
						i+1, new DateTime());
			}catch(IncorrectResultSizeDataAccessException ex){
				logger.log(Level.WARN, "found duplicite record in wholeweek");
			}
			if(contr != null){
				if(t.getId()!= null){
					 newT = busTurnsTimetableDAO.findByDayAndTimetable(contr, days.get(i));
					if(newT == null) continue;
				}else{
					newT = new BusTurnsTimetableVO();
					newT.setWeekDay(days.get(i));
					newT.setTimetable(contr);
			    }
				 newT.setCoupleNumber(t.getCoupleNumber());
				busTurnsTimetableDAO.update(newT);
			}else continue;*/
			
			tim = mapper.map(t.getTimetable(),BusTimetableVO.class);
			//pokud je zaskrtnute policko pro editaci sudeho a licheho tydne naraz tak vztvarim zaznamy pro oba tydny
			if(manageBohtWeeks){
				newT = busTurnsTimetableDAO.findByDayAndTimetable(tim, days.get(i), 1);
				if(newT == null){
					newT = new BusTurnsTimetableVO();
					newT.setWeekDay(days.get(i));
					newT.setTimetable(tim);
				}
				newT.setWeek(1);
				newT.setCoupleNumber(t.getCoupleNumber());
				busTurnsTimetableDAO.update(newT);
				
				newT = busTurnsTimetableDAO.findByDayAndTimetable(tim, days.get(i), 2);
				if(newT == null){
					newT = new BusTurnsTimetableVO();
					newT.setWeekDay(days.get(i));
					newT.setTimetable(tim);
				}
				newT.setWeek(2);
				newT.setCoupleNumber(t.getCoupleNumber());
				busTurnsTimetableDAO.update(newT);
			}else{
				newT = busTurnsTimetableDAO.findByDayAndTimetable(tim, days.get(i), t.getWeek());
				if(newT == null){
					newT = new BusTurnsTimetableVO();
					newT.setWeekDay(days.get(i));
					newT.setTimetable(tim);
				}
				newT.setWeek(t.getWeek());
				newT.setCoupleNumber(t.getCoupleNumber());
				busTurnsTimetableDAO.update(newT);
			}
		}
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED, readOnly = true)
	public BusTurnsTimetableDTO findByDayAndTimetable(BusTimetableDTO timetable, WeekDay day, Integer week) {
		BusTurnsTimetableVO vo = busTurnsTimetableDAO.findByDayAndTimetable(mapper.map(timetable, BusTimetableVO.class), day, week);
		if(vo == null) return null;
		return mapper.map(vo, BusTurnsTimetableDTO.class);
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRED, readOnly = true)
	public BusTurnsTimetableDTO findByDayAndTimetable(BusTimetableDTO timetable, WeekDay day, DateTime validity, Integer week) {
		BusTurnsTimetableVO vo = busTurnsTimetableDAO.findByDayAndTimetable(mapper.map(timetable, BusTimetableVO.class), day, validity, week);
		if(vo == null) return null;
		return mapper.map(vo, BusTurnsTimetableDTO.class);
	}

	@Override
	public BusTurnsTimetableDTO findByConnection(BusConnectionDTO conn) {

		List<WeekDay> days = Arrays.asList(WeekDay.values());
		return findByDayAndTimetable(
				conn.getTimetable(),
				days.get(conn.getDepartureTime().getDayOfWeek()-1),
				DateFormatter.toBeginOfDay(conn.getDepartureTime()),
				DateFormatter.getWeekMod2(conn.getDepartureTime())
				);
	}

}
