package cz.sa.shiftPlanner.servicesImpl;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.dozer.Mapper;
import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import cz.sa.helios.api.HeliosDatabaseID;
import cz.sa.helios.api.Steward;
import cz.sa.shiftPlanner.dao.CityDAO;
import cz.sa.shiftPlanner.dao.LineGroupDAO;
import cz.sa.shiftPlanner.dao.PersonDAO;
import cz.sa.shiftPlanner.dto.CityDTO;
import cz.sa.shiftPlanner.dto.PersonDTO;
import cz.sa.shiftPlanner.entities.CityVO;
import cz.sa.shiftPlanner.entities.LineGroupVO;
import cz.sa.shiftPlanner.entities.LineVO;
import cz.sa.shiftPlanner.entities.PersonVO;
import cz.sa.shiftPlanner.entities.StationVO;
import cz.sa.shiftPlanner.entities.infrastructure.UserDTO;
import cz.sa.shiftPlanner.helios.api.HeliosAPIProvider;
import cz.sa.shiftPlanner.services.PersonService;
import cz.sa.shiftPlanner.services.UserService;
import cz.sa.shiftPlanner.utils.LineGroupCitiesSingl;
import cz.sa.shiftPlanner.utils.PlannerRoles;
import cz.sa.shiftPlanner.utils.ServiceFailureException;

@Service
@Transactional(propagation = Propagation.REQUIRED)
public class PersonServiceImpl implements PersonService {

  private static final Logger log = Logger.getLogger(PersonServiceImpl.class);

  private PersonDAO personDAO;
  private CityDAO cityDAO;
  private LineGroupDAO lineGroupDAO;
  private Mapper mapper;
  private UserService userService;

  @Autowired
  HeliosAPIProvider prov;

  @Autowired
  @Qualifier("lineGroupCities")
  LineGroupCitiesSingl singl;

  public void setUserService(UserService userService) {
    this.userService = userService;
  }

  public void setMapper(Mapper mapper) {
    this.mapper = mapper;
  }

  public void setPersonDAO(PersonDAO personDAO) {
    this.personDAO = personDAO;
  }

  public void setCityDAO(CityDAO cityDAO) {
    this.cityDAO = cityDAO;
  }

  public void setLineGroupDAO(LineGroupDAO lineGroupDAO) {
    this.lineGroupDAO = lineGroupDAO;
  }

  @Override
  @Transactional(propagation = Propagation.REQUIRED, readOnly = true)
  public PersonDTO get(Long id) {
    return mapper.map(personDAO.get(id), PersonDTO.class);
  }

  @Override
  @Transactional(propagation = Propagation.REQUIRED)
  public void remove(PersonDTO person) {
    personDAO.remove(personDAO.get(person.getId()));
  }

  @Override
  @Transactional(propagation = Propagation.REQUIRED, readOnly = true)
  public List<PersonDTO> findAll() {

    List<PersonDTO> list = new ArrayList<PersonDTO>();
    for (PersonVO p : personDAO.findAll()) {
      list.add(mapper.map(p, PersonDTO.class));
    }
    return list;
  }

  @Override
  @Transactional(propagation = Propagation.REQUIRED, readOnly = true)
  public List<PersonDTO> findByName(String name, String surname) {
    List<PersonDTO> list = new ArrayList<PersonDTO>();
    for (PersonVO p : personDAO.findByName(name, surname)) {
      list.add(mapper.map(p, PersonDTO.class));
    }
    return list;
  }

  @Override
  public List<PersonDTO> findAllByTrialDay(DateTime day) {
    List<PersonDTO> list = new ArrayList<PersonDTO>();
    for (PersonVO p : personDAO.findAllByTrialDay(day)) {
      list.add(mapper.map(p, PersonDTO.class));
    }
    return list;
  }

  @Override
  public List<PersonDTO> findByHPP(String jobTime) {
    List<PersonDTO> list = new ArrayList<PersonDTO>();
    for (PersonVO p : personDAO.findByHPP(jobTime)) {
      list.add(mapper.map(p, PersonDTO.class));
    }
    return list;
  }

  @Override
  @Transactional(propagation = Propagation.REQUIRED, readOnly = true)
  public List<PersonDTO> findByLineGroup(Long id) {

    List<StationVO> stations = singl.getStationsByLineGroup(id);
    List<PersonDTO> list = new ArrayList<PersonDTO>();
    for (PersonVO p : personDAO.findByStations(stations)) {
      list.add(mapper.map(p, PersonDTO.class));
    }
    return list;
  }

  @Override
  public List<PersonDTO> findByDepCity(CityDTO city) {

    List<PersonDTO> list = new ArrayList<PersonDTO>();
    for (PersonVO p : personDAO.findByDepCity(mapper.map(city, CityVO.class))) {
      list.add(mapper.map(p, PersonDTO.class));
    }
    return list;
  }

  @Override
  @Transactional(propagation = Propagation.REQUIRED, readOnly = true)
  public PersonDTO findByPersonalNumber(Long number) {

    return mapper.map(personDAO.findByPersonalNumber(number), PersonDTO.class);
  }

  @Override
  public void downloadStewardsFromHelios() {

    stewardsByHeliosID(HeliosDatabaseID.SA_CZ);
    stewardsByHeliosID(HeliosDatabaseID.SA_SR);
    removeOld();
    log.info("method downloadStewardsFromHelios ends");
  }

  private void stewardsByHeliosID(HeliosDatabaseID id) {
    for (Steward st : prov.getBusStewards(id)) {
      PersonVO person = null;
      try {
        person = personDAO.findByPersonalNumber(st.getCislo().longValue());
      }
      catch (Exception ex) {
        log.error(ex);
      }
        //pokud najdu podle PersonalNumber, pak neni co resit a jen udelam update
        if (person != null) {
          copyData(st, person);
          personDAO.update(person);
          log.info("person with number " + person.getPersonalNumber() + " was updated");
          setUserByPerson(person, PlannerRoles.STEWARD.toLowerCase());
          continue;
        }
        //pokud podle PersonalNumber nenajdu, pak zkusim najit podle dalsich parametru a udelam update nebo vytvorim noveho cloveka
        else if (person == null) {
          log.info("person not found by personal number, try findByParamenters");
          person = personDAO.findByParameters(st.getJmeno(), st.getPrijmeni(), st.getEmail(), st.getMobil());
          if (person != null) {
            //TODO zkontrolovat tuto situaci
            UserDTO user = userService.findByPerson(person);
            if (user == null)
              setUserByPerson(person, PlannerRoles.STEWARD.toLowerCase());
            else {
              user.setUsername(String.valueOf(person.getPersonalNumber()));
              userService.update(user);
            }
            copyData(st, person);
            person.setPersonalNumber(st.getCislo().longValue());
            personDAO.update(person);
            log.info("person " + person.getPersonalNumber() + " was updated");
          }
          else {
            person = new PersonVO();
            person.setPersonalNumber(st.getCislo().longValue());
            copyData(st, person);
            personDAO.create(person);
            setUserByPerson(person, PlannerRoles.STEWARD.toLowerCase());
          }
        }
    }
  }

  private void removeOld() {
    for (PersonVO p : personDAO.findAll()) {
      if (p.getDateUpdated() != null) {
        DateTime dt = new DateTime();
        dt.minusDays(7);
        //pokud dateUpdated je starsi nez 7 dnu, pak stevarda smazu
        if (p.getDateUpdated().compareTo(dt) > 0) {
          personDAO.remove(p);
          log.info("Person "+p.toString()+" was removed");
        }
      }else{
        personDAO.remove(p);
        log.info("Person "+p.toString()+" was removed");
      }
    }
  }

  private void copyData(Steward st, PersonVO person) {
    person.setEmail(st.getEmail());
    person.setSurname(st.getPrijmeni());
    person.setName(st.getJmeno());
    person.setClassification(st.getStewKategorie());
    person.setHpp(st.isHlavniPP());
    person.setPhonenumber(st.getMobil());
    person.setWeekWorkTime(st.getTydenniUvazek().intValue());
    person.setDateUpdated(new DateTime());
    //z heliosu dostavam id mest 
    if (st.getVyjezdniMista() != null) {
      for (Integer c_id : st.getVyjezdniMista().getInt()) {
        CityVO c = cityDAO.get(c_id.longValue());
        if(c!=null){
          person.getCity().add(c);
          continue;
        }
        log.error("city is null");
      }
    }
    if(st.getYBusLinky() != null){
      for (Integer l_id : st.getYBusLinky().getInt()) {
        LineVO l = lineGroupDAO.getLine(l_id.longValue());
        if(l!=null){
          person.getLine().add(l);
          continue;
        }
        log.error("line is null");
      }
    }
  }

  /**
   * Pokud user prislusny k person neexistuje, vytvori se novy
   * 
   * @param p
   * @param role
   */
  private void setUserByPerson(PersonVO p, String role) {
    UserDTO user = userService.get(String.valueOf(p.getPersonalNumber()));
    if (user == null) {
      user = new UserDTO();
      user.setPerson(mapper.map(p, PersonDTO.class));
      user.setUsername(String.valueOf(p.getPersonalNumber()));
      user.setRole(role);
      try {
        userService.create(user);
        userService.updatePassword(user, user.getUsername());
      }
      catch (ServiceFailureException ex) {
        throw new IllegalArgumentException(ex);
      }
    }
  }

  @Override
  public void createUsersInDTB() {
    PersonVO admin, helplinka, ondra, zuzka, iva, marketa, pavel, karolina, eliska, denisa;
    //		personDAO.create(admin = setNewPerson("Supervisor", "Supervisor", "", true, 1L, ""));
    //		setUserByPerson(admin, PlannerRoles.SUPERVISOR.toLowerCase());
    //    personDAO.create(helplinka = setNewPerson("Helplinka", "helplinka", "", true, 5L, "helplinka@studentagency.cz"));
    //    setUserByPerson(helplinka, PlannerRoles.HELPLINKA.toLowerCase());
    //		personDAO.create(ondra = setNewPerson("Ondřej", "Bednář", "ondrej.bednar@studentagency.cz", true, 8622L, "605 226 545"));
    //		setUserByPerson(ondra, PlannerRoles.COORDINATOR.toLowerCase());
    //		personDAO.create(iva   = setNewPerson("Ivana", "Kašická", "ivana.kasicka@studentagency.cz", true, 4160L, "539 000 314"));
    //		setUserByPerson(iva, PlannerRoles.SUPERVISOR.toLowerCase());
    //		personDAO.create(zuzka = setNewPerson("Zuzana", "Soukupová Foltýnová", "zuzana.soukupova@studentagency.cz", true, 8304L, "+420 603 198 402"));
    //		setUserByPerson(zuzka, PlannerRoles.COORDINATOR.toLowerCase());
    //		personDAO.create(marketa = setNewPerson("Markéta", "Doležalová", "marketa.dolezalova@studentagency.cz", true, 7565L, "+420 731 598 505"));
    //		setUserByPerson(marketa, PlannerRoles.COORDINATOR.toLowerCase());
    //		personDAO.create(pavel = setNewPerson("Pavel", "Staněk", "pavel.stanek@studentagency.cz", true, 8988L, "+420 731 598 558"));
    //		setUserByPerson(pavel, PlannerRoles.COORDINATOR.toLowerCase());
//    		personDAO.create(karolina = setNewPerson("Karolina", "Jaskova", "karolina.jaskova@studentagency.cz", true, 9725L, null));
//    		setUserByPerson(karolina, PlannerRoles.COORDINATOR.toLowerCase());
    //		personDAO.create(eliska = setNewPerson("Eliška", "Kamicová", "eliska.kamicova@studentagency.cz", true, 4079L, "+420 603 198 413"));
    //		setUserByPerson(eliska, PlannerRoles.COORDINATOR.toLowerCase());
    //		personDAO.create(denisa = setNewPerson("Denisa", "Hniličková", "", true, 6719L, ""));
    //		setUserByPerson(denisa, PlannerRoles.COORDINATOR.toLowerCase());
    log.info("create users end");
  }

  private PersonVO setNewPerson(String name, String surname, String mail, Boolean hpp, Long personalNumber, String phonenumber) {
    PersonVO p = new PersonVO();
    p.setEmail(mail);
    p.setName(name);
    p.setSurname(surname);
    p.setHpp(hpp);
    p.setPersonalNumber(personalNumber);
    p.setClassification("coordinator");
    p.setWeekWorkTime(40);
    p.setPhonenumber(phonenumber);
    return p;
  }

}
