package cz.sa.shiftPlanner.utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import cz.sa.shiftPlanner.dao.BusTimetableDAO;
import cz.sa.shiftPlanner.dto.CityDTO;
import cz.sa.shiftPlanner.entities.BusTimetableVO;
import cz.sa.shiftPlanner.entities.CityVO;
import cz.sa.shiftPlanner.entities.LineGroupVO;
import cz.sa.shiftPlanner.entities.StationVO;

/**
 * @author ToSk
 */

@Service
public class LineGroupCitiesSingl {

	@Autowired 
	@Qualifier("busTimetableDAO")
	private BusTimetableDAO busTimetableDAO;
	
	/**
	 *  klic je line_group_id 
	 *  hodnota je List<StationVO>
	 */
	private Map<Long, List<StationVO>> map = new HashMap<Long, List<StationVO>>();
	
	/**
   *  klic je city_id
   *  hodnota je List<LineGroupVO>
   */
	private Map<Long, List<LineGroupVO>> revertMap = new HashMap<Long, List<LineGroupVO>>();
	
	private void generateMap() {
		
		Map<Long, List<CityVO>> mapLoc = new HashMap<Long, List<CityVO>>();
		List<BusTimetableVO> timetables = busTimetableDAO.findByValidity(new DateTime());
		for(BusTimetableVO vo : timetables){
			if(!mapLoc.containsKey(vo.getLine().getId())){
				mapLoc.put(vo.getLine().getId(), new ArrayList<CityVO>());
			}
		}
		
		Iterator it = mapLoc.entrySet().iterator();
	    while (it.hasNext()) {
	        Map.Entry pairs = (Map.Entry)it.next();
	        
	        List<BusTimetableVO> timet = busTimetableDAO.findByLineGroupId((Long)pairs.getKey(), new DateTime());
	        List<StationVO> value = (ArrayList<StationVO>)pairs.getValue();
	        for(BusTimetableVO vo : timet){
	        	if(!value.contains(vo.getArrivalStation())) value.add(vo.getArrivalStation());
	        	if(!value.contains(vo.getDepartureStation())) value.add(vo.getDepartureStation());
	        }
	        map.put((Long)pairs.getKey(), value);
	    }
	}
	
	private void generateRevertMap() {
		
		Map<Long, List<LineGroupVO>> mapLoc = new HashMap<Long, List<LineGroupVO>>();
		List<BusTimetableVO> timetables = busTimetableDAO.findByValidity(new DateTime());
		for(BusTimetableVO vo : timetables){
			if(!mapLoc.containsKey(vo.getDepartureStation().getId())){
				mapLoc.put(vo.getDepartureStation().getCity().getId(), new ArrayList<LineGroupVO>());
			}
		}
		
		Iterator it = mapLoc.entrySet().iterator();
	    while (it.hasNext()) {
	        Map.Entry pairs = (Map.Entry)it.next();
	        
	        List<BusTimetableVO> timet = busTimetableDAO.findByDepCityId((Long)pairs.getKey(), new DateTime());
	        List<LineGroupVO> value = (ArrayList<LineGroupVO>)pairs.getValue();
	        for(BusTimetableVO vo : timet){
	        	if(!value.contains(vo.getLine()) && vo.getLine() != null)
	        		value.add(vo.getLine());
	        }
	        revertMap.put((Long)pairs.getKey(), value);
	    }
	}

	public Map<Long, List<StationVO>> getMap() {
		if(map.size() == 0) generateMap();
		return map;
	}

	public List<StationVO> getStationsByLineGroup(Long id){
		if(map.size() == 0) generateMap();
		return map.get(id);
	}
	
	public List<LineGroupVO> getLinesByCities(List<CityDTO> cities){
		
		if(revertMap.size() == 0) generateRevertMap();
		Set<LineGroupVO> result = new HashSet<LineGroupVO>();
		for(CityDTO c : cities){
			List<LineGroupVO> list = revertMap.get(c.getId());
			if(list != null)
				result.addAll(list);
		}
		return new ArrayList<LineGroupVO>(result);
	}
}
