package cz.sa.shiftPlanner.utils;

import java.io.Serializable;
import java.util.Calendar;
import java.util.Date;

import org.joda.time.DateTime;
import org.joda.time.LocalDate;

import cz.sa.shiftPlanner.entities.SymbolVO;

/**
 * @author ToSk
 */

public class TimeRange{

	private DateTime upper;
	private DateTime lower;
	
	/*
	 * if values lower and upper is equals each other constructor prolongue both to begin and end of day
	 */
	public TimeRange(DateTime lower, DateTime upper){
		if(lower.compareTo(upper) ==0){
			this.lower = DateFormatter.toBeginOfDay(lower);
			this.upper = DateFormatter.toEndOfDay(upper);
		}else{
			this.lower = lower;
			this.upper = upper;
		}
	}

	
	public boolean includes(DateTime dt){
		
		if(dt.compareTo(upper) <= 0 && dt.compareTo(lower) >= 0) return true;
		return false;
	}
	
	/*
	 * return true if values from and to combine to interval overlap range 
	 */
	public boolean overlap(DateTime from, DateTime to){
		
		//vne intervalu
		if(to.compareTo(lower) <= 0) return false;
		if(from.compareTo(upper) >= 0) return false;
		
		//pres hranici intervalu
		if(from.compareTo(lower) <= 0 && to.compareTo(lower) >= 0) return true;
		if(from.compareTo(upper) <= 0 && to.compareTo(upper) >= 0) return true;
		
		//uvnitr intervalu
		if(from.compareTo(upper) <= 0 && to.compareTo(lower) >= 0) return true;
		return false;
	}
	
	/**
	 * 
	 * @param startDate
	 * @param endDate
	 * @return number of workDays between startDate and endDate include both
	 */
	private int calculateDuration(Date startDate, Date endDate)
  {
    Calendar startCal = Calendar.getInstance();
    startCal.setTime(startDate);
   
    Calendar endCal = Calendar.getInstance();
    endCal.setTime(endDate);
   
    int workDays = 0;
   
    if (startCal.getTimeInMillis() > endCal.getTimeInMillis())
    {
      startCal.setTime(endDate);
      endCal.setTime(startDate);
    }
   
    do
    {
      if (startCal.get(Calendar.DAY_OF_WEEK) != Calendar.SATURDAY && startCal.get(Calendar.DAY_OF_WEEK) != Calendar.SUNDAY)
      {
        if(!SymbolVO.isHoliday(new LocalDate(startCal)))
          workDays++;
      }
      startCal.add(Calendar.DAY_OF_MONTH, 1);
    }
    while (startCal.getTimeInMillis() < endCal.getTimeInMillis());
   
    return workDays;
  }


  public DateTime getUpper() {
    return upper;
  }


  public void setUpper(DateTime upper) {
    this.upper = upper;
  }


  public DateTime getLower() {
    return lower;
  }


  public void setLower(DateTime lower) {
    this.lower = lower;
  }
	
	
}


