package cz.sa.shiftPlanner.ybus.sync;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.joda.time.DateTime;
import org.springframework.stereotype.Service;

import cz.sa.shiftPlanner.entities.LineGroupVO;
import cz.sa.shiftPlanner.entities.LineVO;
import cz.sa.ybus.api.ConnectionsRequest;
import cz.sa.ybus.api.ConnectionsResponse;
import cz.sa.ybus.api.Line;
import cz.sa.ybus.api.LineGroup;
import cz.sa.ybus.api.LineType;
import cz.sa.ybus.api.LinesFilter;
import cz.sa.ybus.api.LinesFilter.LineGroups;
import cz.sa.ybus.api.Valid;

/**
 * @author ToSk
 */

@Service
//@Transactional(propagation = Propagation.REQUIRES_NEW)
public class SyncLineGroups extends SyncAction {

	private static final Logger log = Logger.getLogger(SyncLineGroups.class);
	
	private List<LineGroupVO> getLinesG() {
		List<LineGroup> lines = getService().listLineGroups();
		
		ConnectionsRequest request = factory.createConnectionsRequest();
		ConnectionsResponse response;
		LinesFilter filter = factory.createLinesFilter();
		LineGroups groups = new LineGroups();

		List<LineGroupVO> result = new ArrayList<LineGroupVO>();
		for (LineGroup c : lines) {
			LineGroupVO l = new LineGroupVO();
			l.setCode(c.getCode());
			l.setId(c.getId());
			l.setName(c.getName());
			l.setSuperIntendant(c.getSuperintendant());
			l.setDateUpdated(new DateTime());
			
			groups.getLineGroupId().add(l.getId());
			filter.setLineGroups(groups);
			request.setLinesFilter(filter);
			response = getService().getTimesheetData(request);
			boolean isInter = true;
			for(Line line : response.getLines().getLine()){
				if(line.getLineType().compareTo(LineType.INTERNATIONAL) == 0){
					isInter = false;
				}
			}
			l.setInterstate(isInter);
			groups.getLineGroupId().clear();
			
			result.add(l);
		}
		return result;
	}
	
	private List<Line> getLinesActualValid(){
	  ConnectionsRequest request = factory.createConnectionsRequest();
    ConnectionsResponse response;
    LinesFilter filter = factory.createLinesFilter();
    Valid valid = new Valid();
    valid.setAfter(new DateTime().toLocalDate());
    valid.setBefore(new DateTime().toLocalDate());
    filter.setValid(valid);
    request.setLinesFilter(filter);
    response = getService().getTimesheetData(request);
    return response.getLines().getLine();
	}
	
	private void downloadLines(){
	  for(Line l : getLinesActualValid()){
	    LineVO line = lineGroupDAO.getLine(l.getId());
	    if(line == null){
	      line = new LineVO();
	      line.setId(l.getId());
	      line.setLineGroup(lineGroupDAO.get(l.getLineGroupId()));
	      line.setLineType(cz.sa.shiftPlanner.enums.LineType.valueOf(l.getLineType().toString()));
	      line.setName(l.getName());
	      line.setValidFrom(l.getValidFrom().toDateTimeAtCurrentTime());
	      line.setValidTo(l.getValidTo().toDateTimeAtCurrentTime());
	      lineGroupDAO.saveLine(line);
	    }
	  }
	}
	
	
	private void downloadLineGroups(){
		for(LineGroupVO l : getLinesG()){
			LineGroupVO lLoc =null;
			lLoc = lineGroupDAO.get(l.getId());
			if(lLoc == null) {
				lineGroupDAO.save(l);
				log.info("Save new line group: "+l.toString());
			}
			else{
				lLoc.setCode(l.getCode());
				lLoc.setName(l.getName());
				lLoc.setSuperIntendant(l.getSuperIntendant());
				lLoc.setInterstate(l.isInterstate());
				//manualni oprava dvou hodnot, ktere potrebuji mit jinak nez mi chodi y YBUSu
				if(l.getCode().equalsIgnoreCase("KV"))
				  lLoc.setInterstate(false);
				if(l.getName().equalsIgnoreCase("OSTRAVA"))
				  lLoc.setInterstate(true);
				lLoc.setDateUpdated(new DateTime());
				lineGroupDAO.update(lLoc);
				log.info("Update line group: "+l.toString());
			}
		}
		log.info("update line groups ends");
		System.out.println("end");
		return;
	}
	
	@Override
	public void run() {
//		downloadLineGroups();
//	  downloadLines();
		log.info("run lines");
//		throw new IllegalArgumentException("lines");
	}

	
}


