package cz.sa.shiftPlanner.ybus.sync;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;
import org.joda.time.DateTime;
import org.springframework.stereotype.Service;

import com.google.common.collect.Lists;

import cz.sa.shiftPlanner.entities.BusTimetableVO;
import cz.sa.shiftPlanner.entities.BusTurnsTimetableVO;
import cz.sa.shiftPlanner.entities.LineGroupVO;
import cz.sa.shiftPlanner.entities.RuleVO;
import cz.sa.shiftPlanner.entities.StationVO;
import cz.sa.shiftPlanner.entities.SymbolVO;
import cz.sa.shiftPlanner.entities.TimeCodeVO;
import cz.sa.shiftPlanner.utils.ServiceFailureException;
import cz.sa.shiftPlanner.utils.ServiceNoticeException;
import cz.sa.ybus.api.Changed;
import cz.sa.ybus.api.Connection;
import cz.sa.ybus.api.ConnectionsFilter;
import cz.sa.ybus.api.ConnectionsFilter.Connections;
import cz.sa.ybus.api.ConnectionsRequest;
import cz.sa.ybus.api.ConnectionsResponse;
import cz.sa.ybus.api.Created;
import cz.sa.ybus.api.Deleted;
import cz.sa.ybus.api.ResultCodes;

/**
 * @author ToSk
 */

@Service
//@Transactional(propagation = Propagation.REQUIRES_NEW)
public class SyncTimetable extends SyncAction {

  private static final Logger log = Logger.getLogger(SyncTimetable.class);

  /**
   * 
   * @param before
   * @param after
   * @param auditsOnly
   * @return
   */
  private List<Connection> getConnectionsByCreated(DateTime before, DateTime after, boolean auditsOnly) {
    Created created = factory.createCreated();
    if(after != null)
      created.setAfter(after);
    if(before != null)
      created.setBefore(before);

    // vytvorim request, ktery potrebuje metoda pro vraceni dat
    ConnectionsRequest request = factory.createConnectionsRequest();
    // request specifikuju filtrem co ktereho zadam co me zajima
    ConnectionsFilter filter = factory.createConnectionsFilter();
    filter.setAuditsOnly(auditsOnly);
    ConnectionsResponse response;
    List<Connection> list = new ArrayList<Connection>();

    do {
      filter.setCreated(created);
      request.setConnectionsFilter(filter);
      response = getService().getTimesheetData(request);
      list.addAll(response.getConnections().getConnection());
      //jen kontrola kvuli outOfArrayIndex
      if (list.size() > 5)
        created.setAfter(list.get(list.size() - 2).getFilterDate());
    } while (response.getResult().getCode().compareTo(ResultCodes.EXCEEDED) == 0);

    log.info("Method getConnectionsByCreated end");
    return list;
  }
  
  private List<Connection> getConnectionsByIds(List<Long> ids){
    ConnectionsRequest request = factory.createConnectionsRequest();
    // request specifikuju filtrem co ktereho zadam co me zajima
    ConnectionsFilter filter = factory.createConnectionsFilter();
    ConnectionsResponse response;
    List<Connection> list = new ArrayList<Connection>();
    Connections value = new Connections();
    value.getIds().addAll(ids);
    filter.setConnections(value);

    request.setConnectionsFilter(filter);
    response = getService().getTimesheetData(request);
    list.addAll(response.getConnections().getConnection());

    log.info("Method getConnectionsByIds end");
    return list;
  }

  private List<Connection> getConnectionsByDeleted(DateTime before, DateTime after) {

    Deleted deleted = factory.createDeleted();
    deleted.setAfter(after);
    deleted.setBefore(before);

    // vytvorim request, ktery potrebuje metoda pro vraceni dat
    ConnectionsRequest request = factory.createConnectionsRequest();
    // request specifikuju filtrem co ktereho zadam co me zajima
    ConnectionsFilter filter = factory.createConnectionsFilter();
    ConnectionsResponse response;
    List<Connection> list = new ArrayList<Connection>();

    do {
      filter.setDeleted(deleted);
      request.setConnectionsFilter(filter);
      response = getService().getTimesheetData(request);
      list.addAll(response.getConnections().getConnection());
      if (list.size() > 5)
        deleted.setAfter(list.get(list.size() - 2).getFilterDate());
    } while (response.getResult().getCode().compareTo(ResultCodes.EXCEEDED) == 0);

    log.info("Method getConnectionsByDeleted end");
    return list;
  }

  private List<Connection> getConnectionsByChanged(DateTime before, DateTime after) {

    Changed changed = factory.createChanged();
    changed.setAfter(after);
    changed.setBefore(before);

    // vytvorim request, ktery potrebuje metoda pro vraceni dat
    ConnectionsRequest request = factory.createConnectionsRequest();
    // request specifikuju filtrem co ktereho zadam co me zajima
    ConnectionsFilter filter = factory.createConnectionsFilter();
    ConnectionsResponse response;
    List<Connection> list = new ArrayList<Connection>();

    do {
      filter.setChanged(changed);
      request.setConnectionsFilter(filter);
      response = getService().getTimesheetData(request);
      list.addAll(response.getConnections().getConnection());
      if (list.size() > 5)
        changed.setAfter(list.get(list.size() - 2).getFilterDate());
    } while (response.getResult().getCode().compareTo(ResultCodes.EXCEEDED) == 0);

    log.info("Method getConnectionsByChanged end");
    return list;
  }

  private void downloadTimetablesToUpdate() {
    for (Connection c : getConnectionsByChanged(new DateTime(), new DateTime().minusMonths(10))) {
      if (c != null) {
        BusTimetableVO vo = null;
        vo = busTimetableDAO.get(c.getId());
        try{
          if (vo == null) 
              createNewTimetable(c);
          else 
              updateTimetable(c, vo);
        }catch(ServiceNoticeException ex){
          createNotification("TimetableToUpdate s id="+c.getId() +" skoncilo s chybou: "+ex.getMessage());
        }
      }
    }
    log.info("Timetable download end");
  }

  /*
   * podle copied from zjistim z jakeho timetable novy zaznam vychazi a adekvatne prekopiruju toceni autobusu
   * pokud je copied from null pak jde o nove vytvoreny zaznam, kteremu je treba toceni teprve sestavit
   */
  private void downloadTimetablesToCreate() {
    for (Connection c : getConnectionsByCreated(new DateTime(), new DateTime().withMonthOfYear(1), false)) {
      if (c != null) {
        BusTimetableVO vo = busTimetableDAO.get(c.getId());
        try{
          if (vo == null) 
              createNewTimetable(c);
          else 
              updateTimetable(c, vo);
        }catch(ServiceNoticeException ex){
          createNotification("TimetableToCreate s id="+c.getId() +" skoncilo s chybou: "+ex.getMessage());
        }
      }
    }
    log.info("Timetable download end");
  }
  
  /**
   * Zadam pouze before na new DateTime(). AuditsOnly na true a vemu si pouze ty, ktere maji copied from a u nich udelam kopirovani tocek a pravidel.
   */
  private void downloadTimetablesToUpdateReferencies() {
    for (Connection c : getConnectionsByCreated(new DateTime(), null, true)) {
      if (c != null) {
        BusTimetableVO vo = busTimetableDAO.get(c.getId());
        try{
          if (vo == null) 
              createNewTimetable(c);
          else{ 
              if(c.getCopiedFrom() != null){
                copyReferencies(c, vo);
              }
          }
        }catch(ServiceNoticeException ex){
          createNotification("TimetableToCreate s id="+c.getId() +" skoncilo s chybou: "+ex.getMessage());
        }
      }
    }
    log.info("downloadTimetablesToUpdateReferencies ends");
  }

  private void updateTimetable(Connection c, BusTimetableVO vo) throws ServiceNoticeException {
    StationVO arrC = cityDAO.getStation(c.getArrivalCityId());
    StationVO depC = cityDAO.getStation(c.getDepartureCityId());
    LineGroupVO line = lineGroupDAO.get(c.getLineGroupId());
    if (arrC == null || depC == null) 
      throw new ServiceNoticeException("vyjezdove nebo prijezdove mesto je null");
    if (line == null) 
      throw new ServiceNoticeException("skupina linek je null");
    if(c.getArrivalTime() == null || c.getDepartureTime() == null)
      throw new ServiceNoticeException("vyjezdovy nebo prijezdovy cas je null");
    vo.setArrivalStation(arrC);
    vo.setDepartureStation(depC);
    vo.setLine(line);
    vo.setArrivalTime(c.getArrivalTime().toDateTimeToday());
    vo.setDepartureTime(c.getDepartureTime().toDateTimeToday());
    vo.setNumber(1);
    vo.setValidity_from(c.getValidityFrom().toDateTimeAtCurrentTime());
    vo.setValidity_to(c.getValidityTo().toDateTimeAtCurrentTime());
    TimeCodeVO tcode = timeCodeDAO.get(c.getTimecodeId());
    vo.setTimeCode(tcode);
    busTimetableDAO.update(vo);
    log.info("Timetable " + vo.getId() + " was updated.");
  }

  private void createNewTimetable(Connection c) throws ServiceNoticeException{
    BusTimetableVO vo = new BusTimetableVO();
    vo.setId(c.getId());
    try {
      copyLineGroup(c, vo);
    }
    catch (ServiceNoticeException e) {
      log.info("unwanted lineGroup");
      return;
    }
    try{
      copyCity(c, vo);
    }catch(ServiceFailureException e){
      log.error("city ID in timetable: "+ c.getId()+" is null");
      throw new ServiceNoticeException(e);
    }
    if(c.getArrivalTime() == null || c.getDepartureTime() == null)
      throw new ServiceNoticeException("arrival or departure time is null in timetable: "+ c.getId());
    vo.setArrivalTime(c.getArrivalTime().toDateTimeToday());
    vo.setDepartureTime(c.getDepartureTime().toDateTimeToday());
    vo.setNumber(1);
    vo.setValidity_from(c.getValidityFrom().toDateTimeAtCurrentTime());
    vo.setValidity_to(c.getValidityTo().toDateTimeAtCurrentTime());
    if (c.getSymbols() != null)
      for (Long l : c.getSymbols().getSymbol()) {
        SymbolVO s = timeCodeDAO.getSymbol(l);
        if (s != null && !vo.getSymbols().contains(s))
          vo.getSymbols().add(s);
      }
    TimeCodeVO tcode = timeCodeDAO.get(c.getTimecodeId());
    vo.setTimeCode(tcode);
    busTimetableDAO.save(vo);
    log.info("Timetable " + vo.getId() + " was saved.");
    if (c.getCopiedFrom() != null) {
      copyReferencies(c, vo);
      log.info("Timetable " + vo.getId() + ". Referencies was copied");
    }
  }

  /**
   * Na pokud u noveho jizdniho radu najde tocku, pak nic nedela. Jen pokud nenajde, tak vytvori novou, ktera je ekvivalentni s tockou, ktera byla u stareho jizdniho radu.
   * Stejny postup plati pro pravidla, ktera se take musi zkopirovat = najdu vsechna pravidla u stare tocky. Podivam se ktera uz jsou u nove tocky a ta zkopiruju.
   * @param t
   * @param vo
   */
  private void copyReferencies(Connection c, BusTimetableVO vo) {
    log.info("Timetable "+vo.getId()+". Start copying referencies from "+c.getCopiedFrom());
    List<BusTurnsTimetableVO> turns = busTurnsTimetableDAO.findByTimetable(c.getCopiedFrom());
    for (BusTurnsTimetableVO t : turns) {
      BusTurnsTimetableVO turn = busTurnsTimetableDAO.findByDayAndTimetable(vo, t.getWeekDay(), t.getValidity_from(), t.getWeek());
      if(turn == null){
        turn = mapper.map(t, BusTurnsTimetableVO.class);
        turn.setId(null);
        turn.setTimetable(vo);
        turn.setRules(new HashSet<RuleVO>());
        busTurnsTimetableDAO.create(turn);
        for(RuleVO r : ruleDAO.findByTurn(t)){
          RuleVO rule = ruleDAO.findByWeekAndTurn(turn, r.getWeek());
          if(rule == null){
            rule = mapper.map(r, RuleVO.class);
            rule.setTurn(turn);
            rule.setId(null);
            ruleDAO.create(rule);
          }
        }
      }
    }
  }

  private void copyLineGroup(Connection c, BusTimetableVO vo) throws ServiceNoticeException {
    LineGroupVO line = lineGroupDAO.get(c.getLineGroupId());
    if (line == null) {
      createNotification("Jizdni rad s id=" + c.getId() + " nema odpovidajici skupinu linek.");
      log.error("Jizdni rad s id=" + c.getId() + " nema odpovidajici skupinu linek.");
      throw new IllegalArgumentException("LineGroup is null");
    }
    for (String except : ExceptStrings.exceptions) {
      if (line.getName().toLowerCase().contains(except.toLowerCase()))
        throw new ServiceNoticeException("Line group is unwanted");
    }
    vo.setLine(line);
  }

  private void copyCity(Connection c, BusTimetableVO vo) throws ServiceFailureException{
    StationVO arrC, depC;
    try{
      arrC = cityDAO.getStation(c.getArrivalCityId());
      depC = cityDAO.getStation(c.getDepartureCityId());
    }catch(IllegalArgumentException ex){
      throw new ServiceFailureException(ex);
    }

    if (arrC == null || depC == null) {
      createNotification("Jizdni rad s id=" + c.getId() + " nema odpovidajici vyjezdni nebo prijezdove mesto");
      log.error("Jizdni rad s id=" + c.getId() + " nema odpovidajici vyjezdni nebo prijezdove mesto");
      throw new IllegalArgumentException("vyjezdove nebo prijezdove mesto je null");
    }

    vo.setArrivalStation(arrC);
    vo.setDepartureStation(depC);
  }

  private void downloadTimetablesToDelete() {
    for (Connection c : getConnectionsByDeleted(new DateTime(), new DateTime().minusMonths(1))) {
      if (c != null) {
        BusTimetableVO vo = busTimetableDAO.get(c.getId());
        if (vo != null) {
          busTimetableDAO.remove(vo);
        }
        else {
          log.info("timetable nenalezeno");
        }
      }
    }
  }

  public Set<Long> downloadByIds(Set<Long> ids){ 
    log.info("download timetable by Ids start");
    Set<Long> timIds = new HashSet<Long>();
    if(ids == null || ids.size() == 0)
      return timIds;
    
    for(Connection c : getConnectionsByIds(Lists.newArrayList(ids))){
      try{
        if (busTimetableDAO.get(c.getId()) == null){ 
            createNewTimetable(c);
            timIds.add(c.getId());
        }
      }catch(ServiceNoticeException ex){
        createNotification("TimetableByIds s id="+c.getId() +" skoncilo s chybou: "+ex.getMessage());
      }
    }
    log.info("download timetable by Ids end");
    return timIds;
  }
  
  public void cronRun(){ 
    log.info("cronRun timetable");
  }
  
  @Override
  public void run() {
//    downloadTimetablesToCreate();
//   	downloadTimetablesToUpdate();
    log.info("run timetable");
  }
  
}
