package cz.sa.shiftPlanner.application;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.wicket.ConverterLocator;
import org.apache.wicket.IConverterLocator;
import org.apache.wicket.Page;
import org.apache.wicket.RuntimeConfigurationType;
import org.apache.wicket.Session;
import org.apache.wicket.authroles.authentication.AbstractAuthenticatedWebSession;
import org.apache.wicket.authroles.authentication.AuthenticatedWebApplication;
import org.apache.wicket.authroles.authorization.strategies.role.IRoleCheckingStrategy;
import org.apache.wicket.authroles.authorization.strategies.role.Roles;
import org.apache.wicket.authroles.authorization.strategies.role.metadata.MetaDataRoleAuthorizationStrategy;
import org.apache.wicket.markup.html.WebPage;
import org.apache.wicket.request.Request;
import org.apache.wicket.request.Response;
import org.apache.wicket.spring.injection.annot.SpringComponentInjector;
import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Autowired;

import cz.sa.shiftPlanner.auth.UserRoleCollection;
import cz.sa.shiftPlanner.web.AutoLoginPage;
import cz.sa.shiftPlanner.web.LineGroupViewPage;
import cz.sa.shiftPlanner.web.PersonViewPage;
import cz.sa.shiftPlanner.web.SignIn;
import cz.sa.shiftPlanner.web.auth.PlannerSession;

@org.springframework.stereotype.Component("adminApplication")
public class ShiftPlannerApp extends AuthenticatedWebApplication {

	private static final Logger logger = Logger.getLogger(ShiftPlannerApp.class);
	
	@Autowired
	private UserRoleCollection collection;
	
	@Override
	protected void init() {

		super.init();
	
	    
		getComponentInstantiationListeners().add(new SpringComponentInjector(this));
		getSecuritySettings().setAuthorizationStrategy(new MetaDataRoleAuthorizationStrategy(new MyRoleCheckingStrategy()));
		getMarkupSettings().setDefaultMarkupEncoding("UTF-8");
		getApplicationSettings().setPageExpiredErrorPage(SignIn.class);
		BasicConfigurator.configure();
//		getDebugSettings().setOutputComponentPath(true);
		
		mountPage("lineGroup", LineGroupViewPage.class);
		mountPage("people", PersonViewPage.class);
		
		if(getConfigurationType().compareTo(RuntimeConfigurationType.DEVELOPMENT)==0)
			mountPage("autologin", AutoLoginPage.class);
	}
	
	@Override
	public RuntimeConfigurationType getConfigurationType() {
		
		return RuntimeConfigurationType.DEVELOPMENT;
	}
	
	@Override
	public Session newSession(Request request, Response response) {
		return super.newSession(request, response);
		
	}
	
	@Override
	protected Class<? extends AbstractAuthenticatedWebSession> getWebSessionClass() {

		return PlannerSession.class;
	}

	@Override
	protected Class<? extends WebPage> getSignInPageClass() {

		return SignIn.class;
	}

	@Override
	public Class<? extends Page> getHomePage() {

		return SignIn.class;
	}
	
	@Override
	protected IConverterLocator newConverterLocator() {
	  ConverterLocator locator = (ConverterLocator) super.newConverterLocator();

    locator.set(DateTime.class, DateTimeConverter.INSTANCE);

    return locator;
	}

	private class MyRoleCheckingStrategy implements IRoleCheckingStrategy{

		@Override
		public boolean hasAnyRole(Roles objectRoles) {
//			return true;
			List<String> userRole = new ArrayList<String>(PlannerSession.get().getRoles());
			if(userRole.size() > 1) logger.log(Level.WARN, "More user roles than 1");
			if(userRole.size() == 0) return false;
			for(String objectRole : objectRoles){
				if(collection.checkRoleHierarchy(userRole.get(0), objectRole))
				  return true;
			}
			return false;
		}
		
	}
}
