package cz.sa.shiftPlanner.web;

import java.util.ArrayList;
import java.util.List;

import org.apache.wicket.authroles.authorization.strategies.role.metadata.MetaDataRoleAuthorizationStrategy;
import org.apache.wicket.extensions.ajax.markup.html.tabs.AjaxTabbedPanel;
import org.apache.wicket.extensions.markup.html.tabs.AbstractTab;
import org.apache.wicket.extensions.markup.html.tabs.ITab;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.Model;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.spring.injection.annot.SpringBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Configurable;

import cz.sa.shiftPlanner.auth.UserRoleCollection;
import cz.sa.shiftPlanner.dto.PersonDTO;
import cz.sa.shiftPlanner.entities.infrastructure.UserDTO;
import cz.sa.shiftPlanner.enums.UserRoleHierarchy;
import cz.sa.shiftPlanner.utils.PlannerRoles;
import cz.sa.shiftPlanner.web.auth.PlannerSession;
import cz.sa.shiftPlanner.web.panels.personDetail.AbsencePanel;
import cz.sa.shiftPlanner.web.panels.personDetail.NotesPanel;
import cz.sa.shiftPlanner.web.panels.personDetail.TimeTrialPanel;
import cz.sa.shiftPlanner.web.panels.personDetail.WayContainer;

/**
 * @author ToSk
 */

@Configurable
public class PersonDetailPage extends MenuPage{

  @SpringBean
  private UserRoleCollection collection;
  
	List<ITab> tabs = new ArrayList<ITab>();
	
	public PersonDetailPage(final PageParameters parameters) {
		super(parameters);
		
		final PersonDTO person = personService.get(parameters.get("perId").toLong());
		
		add(new Label("fullName", person.getName() + " "+ person.getSurname()));
		
		AbstractTab notes = new AbstractTab(new Model<String>("Poznamky")) {
      public Panel getPanel(String panelId) {
        return new NotesPanel(panelId, person);
      }
    };
    //pokud ma aktualne prihlaseny uzivatel vyssi nebo stejnou roli jako coordinator, pak uvidi poznamky
    if(collection.checkRoleHierarchy(
          ((PlannerSession)PlannerSession.get()).getLoggedUser().getRole(),
          UserRoleHierarchy.Coordinator.name())){
       tabs.add(notes);
    }
    
		tabs.add(new AbstractTab(new Model<String>("Cesty")) {
			public Panel getPanel(String panelId) {
				return new WayContainer(panelId, person);
			}
		});
		
		tabs.add(new AbstractTab(new Model<String>("Absence")) {
			public Panel getPanel(String panelId) {
				return new AbsencePanel(panelId, person);
			}
		});
		
		tabs.add(new AbstractTab(new Model<String>("Casovky")) {
			public Panel getPanel(String panelId) {
				return new TimeTrialPanel(panelId, person);
			}
		});
		
		add(new AjaxTabbedPanel("tabs", tabs));
	}

	


}


