package cz.sa.shiftPlanner.web;

import java.util.List;

import org.apache.wicket.authroles.authorization.strategies.role.metadata.MetaDataRoleAuthorizationStrategy;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Button;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.spring.injection.annot.SpringBean;
import org.joda.time.DateTime;

import com.google.common.collect.Lists;

import cz.sa.shiftPlanner.dto.BusTimetableDTO;
import cz.sa.shiftPlanner.services.BusTimetableService;
import cz.sa.shiftPlanner.services.LineGroupService;
import cz.sa.shiftPlanner.utils.PlannerRoles;
import cz.sa.shiftPlanner.web.auth.PlannerSession;
import cz.sa.shiftPlanner.web.panels.BusTimetablePanel;
import cz.sa.shiftPlanner.web.wizard.TimetableTurnsWizard;
import cz.sa.shiftPlanner.web.wizard.WizardPage;

public class TimetableViewPage extends MenuPage {
	
	@SpringBean(name = "busTimetableService")
	BusTimetableService busTimetableService;

	@SpringBean(name = "lineGroupService")
	LineGroupService lineService;
	
	Long id;
	
	public TimetableViewPage(final PageParameters parameters) {
		super(parameters);

		if(!parameters.isEmpty()){
			id = parameters.get("id").toLong();
		}
		add(new Label("message", "Jizdni rad linky: "+lineService.get(id).getName()));
		
		/*
		 * vrati List Listu vytvorene z mapy
		 * klice v mape nejsou podstatne, slouzi pouze pri vytvareni mapy
		 */
		IModel<List< List<BusTimetableDTO>>> model = new LoadableDetachableModel<List<List<BusTimetableDTO>>>() {
			@Override
			protected  List<List<BusTimetableDTO>> load(){
				return Lists.newArrayList(busTimetableService.findByLineGroupId(id, PlannerSession.get().getSelectedValidity()).values());
			}
		};
	
		add(new LineViews("times", model));
		
		
		Link link = new Link("turns", new Model("")){
			@Override
			public void onClick() {
				PageParameters param = new PageParameters();
				param.add("page", 1);
				param.add("id", String.valueOf(parameters.get("id")));
				setResponsePage(WizardPage.class, param);
			}
		};
		MetaDataRoleAuthorizationStrategy.authorize(link, RENDER, PlannerRoles.COORDINATOR);
		add(link);
	
	}

	private class LineViews extends ListView<List<BusTimetableDTO>>{
		
		public LineViews(String id, IModel<List< List<BusTimetableDTO>>> model) {
		      super(id, model);
		 }

		@Override
		protected void populateItem(ListItem <List<BusTimetableDTO>> item) {
			item.add(new BusTimetablePanel("lines",item.getModelObject()));
			
		}
	}
	
	
	
}
