package cz.sa.shiftPlanner.web.modalWindow;

import java.util.Date;
import java.util.List;

import org.apache.wicket.Component;
import org.apache.wicket.Session;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.ajax.markup.html.form.AjaxButton;
import org.apache.wicket.extensions.ajax.markup.html.modal.ModalWindow;
import org.apache.wicket.extensions.yui.calendar.DateTimeField;
import org.apache.wicket.markup.html.WebPage;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.FormComponent;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.spring.injection.annot.SpringBean;
import org.joda.time.DateTime;

import cz.sa.shiftPlanner.dto.CityDTO;
import cz.sa.shiftPlanner.dto.PersonDTO;
import cz.sa.shiftPlanner.dto.StationDTO;
import cz.sa.shiftPlanner.dto.WayDTO;
import cz.sa.shiftPlanner.services.CityService;
import cz.sa.shiftPlanner.services.PersonService;
import cz.sa.shiftPlanner.services.WayService;
import cz.sa.shiftPlanner.utils.ServiceNoticeException;

public class WayPersonModal extends WebPage {

  @SpringBean(name = "personService")
  PersonService personService;

  @SpringBean(name = "cityService")
  CityService cityService;

  @SpringBean(name = "wayService")
  WayService wayService;

  private List<PersonDTO> personCategories = personService.findAll();
  private List<StationDTO> cityCategories = cityService.findAllStations();

  private IModel<Date> depOld;
  private IModel<Date> arrOld = new Model<Date>();
  private IModel<Date> depNew = new Model<Date>();
  private IModel<Date> arrNew;
  private WayDTO way = new WayDTO();

  public WayPersonModal(final WayDTO oldWay, final ModalWindow window) {

    Form<WayDTO> form = new Form<WayDTO>("form");
    form.setOutputMarkupId(true);
    
    way.setArrivalStation(oldWay.getArrivalStation());
    way.setArrivalTime(oldWay.getArrivalTime());
   
    depOld = new Model<Date>(oldWay.getDepartureTime().toDate());
    arrNew = new Model<Date>(oldWay.getArrivalTime().toDate());
    FormComponent<PersonDTO> personDD = new DropDownChoice("pCategoryOld", new PropertyModel<PersonDTO>(oldWay, "person"),
        personCategories);
    FormComponent<StationDTO> depCityDD = new DropDownChoice("cCategoryOld", new PropertyModel<StationDTO>(oldWay, "departureStation"),
        cityCategories);
    FormComponent<StationDTO> arrCityDD = new DropDownChoice("cCategory1Old", new PropertyModel<StationDTO>(oldWay, "arrivalStation"),
        cityCategories);

    form.add(new DateTimeField("departureTimeOld", depOld));
    form.add(new DateTimeField("arrivalTimeOld", arrOld));
    form.add(personDD, depCityDD, arrCityDD);

    FormComponent<PersonDTO> personN = new DropDownChoice("pCategoryNew", new PropertyModel<PersonDTO>(way, "person"),
        personCategories);
    FormComponent<StationDTO> depCityN = new DropDownChoice<StationDTO>("cCategoryNew", new PropertyModel<StationDTO>(way, "departureStation"),
        cityCategories);
    FormComponent<StationDTO> arrCityN = new DropDownChoice<StationDTO>("cCategory1New", new PropertyModel<StationDTO>(way, "arrivalStation"),
        cityCategories);

    form.add(new DateTimeField("departureTimeNew", depNew));
    form.add(new DateTimeField("arrivalTimeNew", arrNew));
    form.add(personN, depCityN, arrCityN);

    add(form);

    form.add(new AjaxButton("closeOK") {
      @Override
      public void onSubmit(AjaxRequestTarget target, Form form) {
        way.setArrivalTime(new DateTime(arrNew.getObject()));
        way.setDepartureTime(new DateTime(depNew.getObject()));
        way.setConnection(oldWay.getConnection());
        way.setDelay(0);
        way.setIsForeign(!way.getArrivalStation().getCity().getCountry().equalsIgnoreCase("cz"));
        try {
          wayService.createUseWayTime(way);
        }
        catch (ServiceNoticeException e) {
          Session.get().info(e.getMessage());
        }

        oldWay.setArrivalTime(new DateTime(arrOld.getObject()));
        oldWay.setDepartureTime(new DateTime(depOld.getObject()));
        oldWay.setDelay(0);
        oldWay.setIsForeign(!oldWay.getArrivalStation().getCity().getCountry().equalsIgnoreCase("cz"));
        wayService.remove(oldWay);
        oldWay.setId(null);
        try {
          wayService.createUseWayTime(oldWay);
        }
        catch (ServiceNoticeException e) {
          Session.get().info(e.getMessage());
        }
        window.close(target);
      }
    });

    form.add(new AjaxLink<Void>("close") {

      @Override
      public void onClick(AjaxRequestTarget target) {
        window.close(target);
      }

    });
  }
}