package cz.sa.shiftPlanner.web.notification;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.apache.wicket.extensions.markup.html.repeater.util.SortParam;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.repeater.data.IDataProvider;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;

import com.google.common.collect.Iterables;
import com.google.common.collect.Iterators;
import com.google.common.primitives.Ints;

import cz.sa.shiftPlanner.dto.NotificationDTO;

/**
 *
 * @author    ToSk
 * @version   1.0
 * @created   5. 10. 2013
 */

public abstract class LoadableDataProvider<T, S> extends SortableDataProvider<T, S> {

  private transient Collection<T> col;

  public LoadableDataProvider() {
    super();
  }

  public LoadableDataProvider(S sortProperty, boolean asc) {
    super();
    setSort(new SortParam<S>(sortProperty, asc));
  }

  @Override
  public Iterator<? extends T> iterator(long first, long count) {
    init();

    if (col == null) {
      return Iterators.emptyIterator();
    }

    if (first == 0 && count == col.size()) {
      return col.iterator();
    }
    else {
      int intFirst = Ints.checkedCast(first);
      int intCount = Ints.checkedCast(count);
      return Iterables.limit(Iterables.skip(col, intFirst), intCount).iterator();
    }
  }

  @Override
  public long size() {  
    init();
    return col != null ? col.size() : 0;
  }
  
  public final Collection<T> get() {
    init();
    return col;
  }

  private void init() {
    if (col == null) {
      col = load();
    }
  }

  public final void reset() {
    col = null;
  }

  protected abstract Collection<T> load();

  @Override
  public void detach() {   
    super.detach();
    col = null;
  }

}
