package cz.sa.shiftPlanner.web.notification;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.ajax.markup.html.form.AjaxButton;
import org.apache.wicket.authroles.authorization.strategies.role.metadata.MetaDataRoleAuthorizationStrategy;
import org.apache.wicket.extensions.ajax.markup.html.modal.ModalWindow;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.OrderByBorder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.DataTable;
import org.apache.wicket.extensions.markup.html.repeater.data.table.DefaultDataTable;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.ISortableDataProvider;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortParam;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.image.Image;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.list.AbstractItem;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.navigation.paging.PagingNavigator;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.markup.repeater.data.DataView;
import org.apache.wicket.markup.repeater.data.IDataProvider;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.request.resource.ContextRelativeResource;
import org.apache.wicket.spring.injection.annot.SpringBean;

import cz.sa.shiftPlanner.dto.NotificationDTO;
import cz.sa.shiftPlanner.dto.NotificationLoadDTO;
import cz.sa.shiftPlanner.enums.NotifType;
import cz.sa.shiftPlanner.services.NotificationService;
import cz.sa.shiftPlanner.utils.DateFormatter;
import cz.sa.shiftPlanner.utils.PlannerRoles;

/**
 * ToSk
 */

public class NotificationContentPanel extends Panel implements Serializable{

  @SpringBean(name = "notificationService")
  NotificationService notificationService;

  private NotificationProvider notifProvider;
  private Form form ;
  private NotificationLoadDTO load = new NotificationLoadDTO();

  public NotificationContentPanel(String id, NotifType notifType) {
    super(id);

    this.load.setType(notifType);
    notifProvider = new NotificationProvider();
    renderAll();
  }

  private void renderAll() {

    form= new Form("form");
    form.setOutputMarkupId(true);
    DataTable dataTable = new DefaultDataTable<NotificationDTO, String>("dataTable", columns(), notifProvider, 20);
    form.add(dataTable);
    add(form);

    form.add(new AjaxLink("findAll") {
      @Override
      public void onClick(AjaxRequestTarget target) {
        notifProvider.detach();
        load.setFindAll(true);
        target.add(form);
      }
    });
    
    AjaxLink read = new AjaxLink("readAll") {
      @Override
      public void onClick(AjaxRequestTarget target) {
        for(NotificationDTO n : notifProvider.load()){
          n.setReaded(true);
          notificationService.update(n);
        }
        load.setFindAll(false);
        target.add(form);
      }
    };
    
    AjaxLink delete = new AjaxLink("deleteAll") {
      @Override
      public void onClick(AjaxRequestTarget target) {
        load.setFindAll(true);
        for(NotificationDTO n : notifProvider.load()){
          n.setReaded(true);
          notificationService.remove(n);
        }
        target.add(form);
      }
    };
    MetaDataRoleAuthorizationStrategy.authorize(delete, RENDER, PlannerRoles.COORDINATOR);
    MetaDataRoleAuthorizationStrategy.authorize(read, RENDER, PlannerRoles.COORDINATOR);
    form.add(delete, read);
    
  }

  private List<? extends IColumn<NotificationDTO, String>> columns() {
    List<IColumn<NotificationDTO, String>> columns = new ArrayList<IColumn<NotificationDTO, String>>();
    columns.add(new PropertyColumn<NotificationDTO, String>(new Model<String>("Vytvoreno"), "created", "created"));
    columns.add(new PropertyColumn<NotificationDTO, String>(new Model<String>("Text"), "text", "text"));
    columns.add(new PropertyColumn<NotificationDTO, String>(new Model<String>("Vytvoreno kym"), "person"));
    columns.add(new AbstractColumn<NotificationDTO, String>(new Model<String>(""))
        {
            @Override
            public void populateItem(Item<ICellPopulator<NotificationDTO>> cellItem, String componentId, final IModel<NotificationDTO> rowModel) {
              cellItem.add(new NotificationImagePanel(componentId, rowModel, form));
            }
        });
    return columns;
  }

  private class NotificationProvider extends LoadableDataProvider<NotificationDTO, String> {

    public NotificationProvider() {
      super("created", true);
    }

    @Override
    protected List<NotificationDTO> load() {
      SortParam<String> sort = getSort();
      load.setAscending(sort.isAscending());
      load.setSortProperty(sort.getProperty());
      return notificationService.findSelectedBy(load);
    }

    @Override
    public IModel<NotificationDTO> model(NotificationDTO object) {
      return new Model<NotificationDTO>(object);
    }
  }

}
