package cz.sa.shiftPlanner.web.panels;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.MarkupContainer;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.ajax.markup.html.form.AjaxCheckBox;
import org.apache.wicket.extensions.ajax.markup.html.AjaxEditableMultiLineLabel;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.CheckBox;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.list.AbstractItem;
import org.apache.wicket.markup.html.panel.Fragment;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.RepeatingView;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.spring.injection.annot.SpringBean;
import org.joda.time.DateTime;

import cz.sa.shiftPlanner.dto.BusTurnsTimetableDTO;
import cz.sa.shiftPlanner.dto.PersonDTO;
import cz.sa.shiftPlanner.dto.RuleDTO;
import cz.sa.shiftPlanner.dto.WayDTO;
import cz.sa.shiftPlanner.enums.WeekDay;
import cz.sa.shiftPlanner.services.BusTurnsTimetableService;
import cz.sa.shiftPlanner.services.RuleService;
import cz.sa.shiftPlanner.utils.ServiceNoticeException;

public class BusRulesPanel extends Panel {

  @SpringBean(name = "busTurnsTimetableService")
  BusTurnsTimetableService busTurnsTimetableService;

  @SpringBean(name = "ruleService")
  RuleService ruleService;

  private WebMarkupContainer container;
  private IModel<List<BusTurnsTimetableDTO>> lines;
  private IModel<PersonDTO> person;
  private Integer week;
  private BusTurnsTimetableDTO turnB;
  private BusTurnsTimetableDTO turnA;
  private CheckBox checkAll;

  /**
   * 
   * Trida se chova tak, ze vsechny ajax akce jsou navazany na container z konstruktoru.
   *  Aby pri kazde obnove nedochazelo k hromadnemu nacitani dat z databaze je vse reseno pouze na aplikacni urovni:
   * create a remove pridavaji vytvorene/smazane objekty primo do kolekci. Tuto funkcionalitu v pripade create zajistuje servisa {@link RuleService}
   */
  public BusRulesPanel(String id, IModel<List<BusTurnsTimetableDTO>> lines, WeekDay day, Integer week, IModel<PersonDTO> person, WebMarkupContainer container) {
    super(id);

    this.container = container;
    this.lines = lines;
    this.person = person;
    this.week = week;
    turnB = lines.getObject().get(0);
    turnA = lines.getObject().get(lines.getObject().size() - 1);
    showOneDirection();
  }

  private void showOneDirection() {

    Form form = new Form("form");
    add(form);
    RepeatingView rows = new RepeatingView("rows");
    form.add(rows);

    add(new Label("busNumber", "BUS " + new PropertyModel<Integer>(turnB, "coupleNumber").getObject()));
    add(new AjaxEditableMultiLineLabel("noteBefore", new PropertyModel(turnB, "noteBefore")){
      @Override
      protected void onSubmit(AjaxRequestTarget target) {
        super.onSubmit(target);
        busTurnsTimetableService.update(turnB);
      }
    });

    add(new AjaxEditableMultiLineLabel("noteAfter", new PropertyModel(turnA, "noteAfter")){
      @Override
      protected void onSubmit(AjaxRequestTarget target) {
        super.onSubmit(target);
        busTurnsTimetableService.update(turnA);
      }
    });

    checkAll = new AjaxCheckBox("checkAll", Model.of(Boolean.FALSE)) {
      @Override
      protected void onUpdate(AjaxRequestTarget target) {
        ruleService.createAll(lines.getObject(), week, person.getObject());
        target.add(container);
      }
    };
    add(checkAll);
    
    AjaxLink removeAll = new AjaxLink("removeAll") {
      @Override
      public void onClick(AjaxRequestTarget target) {
        for (BusTurnsTimetableDTO busT : lines.getObject()) {
          if (busT.getRules().size() >= 1) {
            RuleDTO rule = ruleService.findByWeekAndTurn(busT, week);
            if (rule != null) {
              ruleService.remove(rule);
              busT.getRules().remove(rule);
            }
          }
        }
        target.add(container);
      }
    };
    add(removeAll);

    for (int i = 0; i < 3; i++) {
      AbstractItem rowsItem = new AbstractItem(rows.newChildId());

      RepeatingView cols = new RepeatingView("cols");
      rowsItem.add(cols);

      switch (i) {

        case 0: {

          for (BusTurnsTimetableDTO busT : lines.getObject()) {

            AbstractItem colsItem = new AbstractItem(cols.newChildId());

            colsItem.add(new MyFragment("fragmentWrap", "textFragment", this, "odj: "
                + busT.getTimetable().getDepartureStation().getCity().getCityName()).add(new AttributeModifier("title", busT
                .getTimetable().getDepartureStation().getStationName())));
            cols.add(colsItem);

            colsItem = new AbstractItem(cols.newChildId());
            colsItem.add(new MyFragment("fragmentWrap", "textFragment", this, "prij: "
                + busT.getTimetable().getArrivalStation().getCity().getCityName()).add(new AttributeModifier("title", busT
                .getTimetable().getArrivalStation().getStationName())));
            cols.add(colsItem);
          }
          rows.add(rowsItem);
          break;
        }
        case 1: {

          for (BusTurnsTimetableDTO busT : lines.getObject()) {

            AbstractItem colsItem = new AbstractItem(cols.newChildId());

            colsItem.add(new MyFragment("fragmentWrap", "textFragment", this,
                dateFormat(busT.getTimetable().getDepartureTime())));
            cols.add(colsItem);

            colsItem = new AbstractItem(cols.newChildId());
            colsItem
                .add(new MyFragment("fragmentWrap", "textFragment", this, dateFormat(busT.getTimetable().getArrivalTime())));

            cols.add(colsItem);
          }
          rows.add(rowsItem);
          break;
        }
        case 2: {

          for (final BusTurnsTimetableDTO busT : lines.getObject()) {

            AbstractItem colsItem = new AbstractItem(cols.newChildId());
            colsItem.add(new AttributeModifier("colspan", 2));

            Fragment fragment;
            RuleDTO rule = null;//= ruleService.findByWeekAndTurn(busT, week);
            for(RuleDTO r : busT.getRules()){
              if(r.getWeek() == week){
                rule = r;
                break;
              }
            }

            AjaxLink delete = new AjaxLink("crossLink") {
              @Override
              public void onClick(AjaxRequestTarget target) {
                RuleDTO r = ruleService.findByWeekAndTurn(busT, week);
                ruleService.remove(r);
                busT.getRules().remove(r);
                target.add(container);
              }
            };

            if (rule == null) {

              fragment = new Fragment("fragmentWrap", "checkFragment", this);
              CheckBox box = new CheckBox("checkbox", Model.of(Boolean.FALSE));
              box.add(new AjaxFormComponentUpdatingBehavior("click") {
                @Override
                protected void onUpdate(AjaxRequestTarget target) {
                  ruleService.create(busT, week, person.getObject());
                  target.add(container);
                }
              });
              fragment.add(box);
              colsItem.add(new AttributeModifier("style", "background-color:#FFCCCC;"));
            }
            else {

              fragment = new MyFragment("fragmentWrap", "labelFragment", this, rule.getPerson().toString());
              fragment.add(delete);
            }
            colsItem.add(fragment);

            cols.add(colsItem);
          }
          rows.add(rowsItem);
          break;
        }
      }
    }
  }

  private class MyFragment extends Fragment {

    private static final long serialVersionUID = -915399923825573130L;

    public MyFragment(String id, String markupId, MarkupContainer markupProvider, String string) {
      super(id, markupId, markupProvider);

      if (markupId.equals("labelFragment")) {

        add(new Label("label", string));
      }
      if (markupId.equals("textFragment")) {

        add(new Label("label", string));
      }
    }
  }

  private String dateFormat(DateTime time) {
    String sTime;
    sTime = String.format("%02d", time.getHourOfDay()) + " : " + String.format("%02d", time.getMinuteOfHour());
    return sTime;
  }

}
