package cz.sa.shiftPlanner.web.panels;

import java.util.ArrayList;
import java.util.List;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.Component;
import org.apache.wicket.MarkupContainer;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.ajax.markup.html.form.AjaxCheckBox;
import org.apache.wicket.authroles.authorization.strategies.role.metadata.MetaDataRoleAuthorizationStrategy;
import org.apache.wicket.extensions.ajax.markup.html.AjaxEditableLabel;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.list.AbstractItem;
import org.apache.wicket.markup.html.panel.Fragment;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.markup.repeater.data.DataView;
import org.apache.wicket.markup.repeater.data.ListDataProvider;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.spring.injection.annot.SpringBean;

import cz.sa.shiftPlanner.dto.BusConnectionDTO;
import cz.sa.shiftPlanner.dto.PersonDTO;
import cz.sa.shiftPlanner.dto.WayDTO;
import cz.sa.shiftPlanner.services.BusConnectionService;
import cz.sa.shiftPlanner.services.WayService;
import cz.sa.shiftPlanner.utils.DateFormatter;
import cz.sa.shiftPlanner.utils.PlannerRoles;
import cz.sa.shiftPlanner.utils.ServiceNoticeException;

/**
 * @author ToSk
 */

public class BusSupportPanel extends Panel {

	@SpringBean(name = "wayService")
	WayService wayService;
	
	@SpringBean(name = "busConnectionService")
	BusConnectionService busConnectionService;

	
	private List<BusConnectionDTO> suppModel = new ArrayList<BusConnectionDTO>();
	private IModel<PersonDTO> person;
	private WebMarkupContainer listContainer;
	private List<BusConnectionDTO> listToSave = new ArrayList<BusConnectionDTO>();
	
	public BusSupportPanel(String id, IModel<List<BusConnectionDTO>> suppModel, IModel<PersonDTO> person, WebMarkupContainer listContainer) {
		super(id, suppModel);

		this.suppModel = suppModel.getObject();
		this.listContainer=listContainer;
		this.person = person;
		renderDataTable();
	}

	private void renderDataTable() {
		Form form = new Form("suppForm");
		form.setOutputMarkupId(true);
		add(form);
		
		form.add(new Label("supportType", "Posily"));
		Component header = new AbstractItem("header");
		form.add(header);

		ListDataProvider<BusConnectionDTO> dp = new ListDataProvider<BusConnectionDTO>(suppModel);
		final DataView<BusConnectionDTO> dataView = new DataView<BusConnectionDTO>("repeating", dp) {
			private static final long serialVersionUID = 1L;

			@Override
			protected void populateItem(final Item<BusConnectionDTO> item) {
				item.add(new Label("depTime", DateFormatter.dateFormatToTime(item.getModelObject().getDepartureTime())));
				item.add(new Label("arrCity", item.getModelObject().getTimetable().getArrivalStation().getCity().getCityName() + " ("+ item.getModelObject().getStandard().substring(0, 3)+")")
						.add(new AttributeModifier("title", item.getModelObject().getTimetable().getArrivalStation().getStationName())));
				renderFragment(item, "stew");
				renderFragment(item, "clearance");
				item.add(new AjaxEditableLabel("clean", new PropertyModel(item.getModelObject(), "note")){
					@Override
					protected void onSubmit(AjaxRequestTarget target) {
						super.onSubmit(target);
						busConnectionService.update(item.getModelObject());
					}
					
					@Override
					protected String defaultNullLabel() {
						return "-";
					}
				});

				item.add(new AttributeModifier("class", (item.getIndex() % 2 == 1) ? "even" : "odd"));
				MetaDataRoleAuthorizationStrategy.authorize(item, ENABLE, PlannerRoles.COORDINATOR);
			}

		};
		form.add(dataView);
		
		
	}
	
	/**
	 * rozhoduje se co se ma vykreslit. jestli check box nebo label
	 */
	private void renderFragment(final Item<BusConnectionDTO> item, final String frId) {
		
		//krizek na pripadne mazani stewardky
		AjaxLink delete = new AjaxLink("crossLink") {
            @Override
            public void onClick(AjaxRequestTarget target) {
                WayDTO way = item.getModelObject().getWays().get(0);
                wayService.remove(way, way.isClearance());
                item.getModelObject().getWays().clear();
                target.add(listContainer);
            }
        };
        MetaDataRoleAuthorizationStrategy.authorize(delete, RENDER, PlannerRoles.COORDINATOR);
        
		Fragment fragment = new MyFragment(frId, "textFragment", this, "");
		
		//pokud je cesta neobsazena je vykreslen check box
		if(item.getModelObject().getWays().size() == 0){
			
			fragment = new Fragment(frId, "checkFragment", this);
			fragment.add(new AjaxCheckBox("checkbox", new Model<Boolean>()){
				
				protected void onUpdate(AjaxRequestTarget target) {
//					try {
//						WayDTO way = wayService.create(item.getModelObject(), person.getObject(), true);
//						item.getModelObject().getWays().add(way);
//					} catch (ServiceNoticeException e) {}
					
					//nastavi atribut v busConnection, ktery pak slouzi pro identifikaci jestli jde jen o odbaveni nebo o plnohodnotnou cestu
					if(frId.equals("stew")){
						item.getModelObject().setClearance(false);
					}else{
						item.getModelObject().setClearance(true);
					}
					listToSave.add(item.getModelObject());
					//target.add(listContainer);
				}
			});
		//pokud jiz je cesta obsazena vykresli se label se jmenem stevarda a krizkem pro smazani
		}else if(item.getModelObject().getWays().size() == 1){
			
			//pokud jsem ve sloupci odbaveni a stevard je prihlaseny na odbaveni, pak se vykresli jen "" stejne to plati pro sloupce odbaveni
			if((frId.equals("stew") && item.getModelObject().getWays().get(0).isClearance()) || ((frId.equals("clearance") && !item.getModelObject().getWays().get(0).isClearance())))
				fragment = new MyFragment(frId, "textFragment", this, "");
			else{
				fragment = new MyFragment(frId, "labelFragment", this, item.getModelObject().getWays().get(0).getPerson().getSurname());
				fragment.add(delete);
			}
		}
		item.add(fragment);
	}
	
	private class MyFragment extends Fragment{

		public MyFragment(String id, String markupId, MarkupContainer markupProvider, String string) {
			super(id, markupId, markupProvider);
			
			if(markupId.equals("labelFragment")){
			
				add(new Label("label", string));
			}
			if(markupId.equals("textFragment")){
				
				add(new Label("label", string));
			}
		}
	}

	public void save(){
		for(BusConnectionDTO c : listToSave){
			try {
				c.getWays().add(wayService.create(c, person.getObject(), c.isClearance(), true));
			} catch (ServiceNoticeException e) {}
		}
		listToSave.clear();
	}
}
