package cz.sa.shiftPlanner.web.panels;

import java.util.ArrayList;
import java.util.List;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxEventBehavior;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.attributes.AjaxCallListener;
import org.apache.wicket.ajax.attributes.AjaxRequestAttributes;
import org.apache.wicket.ajax.attributes.IAjaxCallListener;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.ajax.markup.html.form.AjaxButton;
import org.apache.wicket.authroles.authorization.strategies.role.metadata.MetaDataRoleAuthorizationStrategy;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.ListChoice;
import org.apache.wicket.markup.html.list.AbstractItem;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.RepeatingView;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.Request;
import org.apache.wicket.request.cycle.RequestCycle;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.spring.injection.annot.SpringBean;
import org.joda.time.DateTime;

import cz.sa.shiftPlanner.dto.BusConnectionDTO;
import cz.sa.shiftPlanner.dto.PersonDTO;
import cz.sa.shiftPlanner.dto.StationDTO;
import cz.sa.shiftPlanner.dto.TimeTrialDTO;
import cz.sa.shiftPlanner.pcomponent.PFeedbackPanel;
import cz.sa.shiftPlanner.services.PersonService;
import cz.sa.shiftPlanner.services.PersonUtilsService;
import cz.sa.shiftPlanner.utils.PlannerRoles;
import cz.sa.shiftPlanner.utils.Tuple;
import cz.sa.shiftPlanner.web.MO.WayMO;
import cz.sa.shiftPlanner.web.auth.PlannerSession;

/**
 * @author ToSk
 */

public class CentralWaysPanel extends Panel {

  @SpringBean(name = "personService")
  PersonService personService;

  @SpringBean(name = "personUtilsService")
  PersonUtilsService personUtilsService;

  private List<WaysPanel> panels = new ArrayList<WaysPanel>();
  private WebMarkupContainer listContainer;
  private IModel<PersonDTO> personModel;
  private List<PersonDTO> categories;
  private TimeTrialDTO ttDTO = new TimeTrialDTO();

  public CentralWaysPanel(String id, final WayMO wayMO, PageParameters parameters) {
    super(id);

    listContainer = new WebMarkupContainer("container");
    personModel = new Model<PersonDTO>();
    listContainer.setOutputMarkupId(true);

    PropertyModel<String> userNumber = new PropertyModel<String>(this, "session.userNumber");
    if (PlannerSession.getLoggedUser().getRole().equals(PlannerRoles.STEWARD.toLowerCase())) {
      personModel.setObject(personService.findByPersonalNumber(Long.valueOf(userNumber.getObject())));
    }
    categories = personService.findByLineGroup(parameters.get("id").toLong());
    
    Component personForm = renderPersonForm();
    listContainer.add(personForm);

    //tocky
    LineViews views = new LineViews("times", wayMO.getConnections(), personModel);
    //posily
    final BusSupportPanel support = new BusSupportPanel("support", wayMO.getSupports(), personModel, listContainer);
    support.setOutputMarkupId(true);

    //pocet odpracovanych hodin a mesicni limit
    final IModel<Integer> sum = new LoadableDetachableModel<Integer>() {
      @Override
      protected Integer load() {
        return personUtilsService.sumMonthHours(personModel.getObject(), wayMO.getTime().getMonthOfYear(),
            new DateTime().getYear()) / 60;
      }
    };
    final IModel<Long> limit = new LoadableDetachableModel<Long>() {
      @Override
      protected Long load() {
        return personUtilsService.limitMonthHours(personModel.getObject(), wayMO.getTime().getMonthOfYear(),
            new DateTime().getYear()) / 60;
      }
    };

    final Label label = new Label("workHours", sum);
    label.setOutputMarkupId(true);
    checkHoursLimit(sum, limit, label);

    //casovky
    Label trialLabel = new Label("trialLabel", "Casovky");
    final List<TimeTrialDTO> choices = personUtilsService.findAllByDay(wayMO.getTime());
    if (choices.size() > 0)
      ttDTO = choices.get(0);
    final ListChoice<TimeTrialDTO> trials = new ListChoice<TimeTrialDTO>("timeTrials", new PropertyModel<TimeTrialDTO>(this,
        "ttDTO"), choices, 5);
    trials.add(new AjaxFormComponentUpdatingBehavior("click") {
      @Override
      protected void onUpdate(AjaxRequestTarget target) {
        ttDTO = trials.getModelObject();
      }
    });
    trials.add(new AjaxEventBehavior("keyup") {
      @Override
      protected void updateAjaxAttributes(AjaxRequestAttributes attributes) {
        super.updateAjaxAttributes(attributes);

        IAjaxCallListener listener = new AjaxCallListener() {
          @Override
          public CharSequence getPrecondition(Component component) {
            //hodnoti zda je se javascript (onEvent) zavola, reaguje pouze na zmacknuti 'delete'
            return "var keycode = Wicket.Event.keyCode(attrs.event);" + "if ((keycode == 46))" + "    return true;" + "else"
                + "    return false;";
          }
        };
        attributes.getAjaxCallListeners().add(listener);

        //Append the pressed keycode to the ajaxrequest 
        attributes.getDynamicExtraParameters().add(
            "var eventKeycode = Wicket.Event.keyCode(attrs.event);" + "return {keycode: eventKeycode};");

        //whithout setting, no keyboard events will reach any inputfield
        attributes.setAllowDefault(true);
      }

      @Override
      protected void onEvent(AjaxRequestTarget target) {
        final Request request = RequestCycle.get().getRequest();
        final String jsKeycode = request.getRequestParameters().getParameterValue("keycode").toString("");
        personUtilsService.remove(ttDTO);
        choices.remove(ttDTO);
        target.add(listContainer);
      }
    });
    trials.setOutputMarkupId(true);
    MetaDataRoleAuthorizationStrategy.authorize(trials, RENDER, PlannerRoles.COORDINATOR);
    MetaDataRoleAuthorizationStrategy.authorize(trialLabel, RENDER, PlannerRoles.COORDINATOR);

    //tlacitko ulozit
    AjaxButton save = new AjaxButton("save") {
      protected void onSubmit(AjaxRequestTarget target, Form form) {
        support.save();
        checkHoursLimit(sum, limit, label);
        panels = new ArrayList<WaysPanel>();
        target.add(listContainer, label);
      }
    };
    listContainer.add(new PFeedbackPanel("feedbackPanel").setOutputMarkupId(true));
    listContainer.add(label, save, views, support, trials, trialLabel);
    listContainer.add(new Label("limitHours", limit));
    add(listContainer);

  }

  private Component renderPersonForm() {
    personModel.setObject(categories.get(0));
    Form<PersonDTO> personForm = new Form<PersonDTO>("personForm");
    final DropDownChoice<PersonDTO> choice = new DropDownChoice<PersonDTO>("personSelect", personModel, categories);
    choice.add(new AjaxFormComponentUpdatingBehavior("onchange") {
      @Override
      protected void onUpdate(AjaxRequestTarget target) {
        target.add(listContainer);
      }
    });
    MetaDataRoleAuthorizationStrategy.authorize(choice, RENDER, PlannerRoles.COORDINATOR);
    personForm.add(choice);
    return personForm;
  }

  private void checkHoursLimit(final IModel<Integer> sum, final IModel<Long> limit, final Label label) {
    if (sum.getObject() >= limit.getObject())
      label.add(new AttributeModifier("style", "background-color: #FFCC66"));
    else
      label.add(new AttributeModifier("style", "background-color: white"));
  }

  private class LineViews extends ListView<List<BusConnectionDTO>> {

    private IModel<PersonDTO> person;
    private Tuple<StationDTO, StationDTO> recentSt = null;
    private boolean isEmpty = false;
    private int round = 0;
    private WebMarkupContainer firstHeader = null;

    public LineViews(String id, IModel<List<List<BusConnectionDTO>>> model, IModel<PersonDTO> person) {
      super(id, model);
      this.person = person;
    }
    @Override
    protected void onBeginPopulateItem(ListItem<List<BusConnectionDTO>> item) {
      super.onBeginPopulateItem(item);
      if(recentSt == null){
        recentSt = new Tuple<StationDTO, StationDTO>(
            item.getModelObject().get(0).getTimetable().getDepartureStation(), 
            item.getModelObject().get(0).getTimetable().getArrivalStation());
      }
    }

    /**
     * pokud je round==0 vytvori hlavicku o 10ti polich, kde se stridaji vyjezdni a prijezdni mesta vzdy po dvou
     * pokud je round!= 0 vytvori prazdnou hlavicku, ktere nastavi class, ktera ji schova jak viditelne, tak pozicne
     * prvni hlavicka je pridana do parametru a je nastavena na plovouci (fixed)
     * @param item
     */
    private void afterPopulateItem(ListItem<List<BusConnectionDTO>> item) {
      super.onBeginPopulateItem(item);
      WebMarkupContainer headerWrap = new WebMarkupContainer("headerWrap");
      RepeatingView header = new RepeatingView("header");
      headerWrap.add(new AttributeModifier("class", "headerTableRelative"));
      if(round == 0){
        BusConnectionDTO dto = item.getModelObject().get(0);
        renderHeaderCell(header, dto.getTimetable().getDepartureStation());
        for (int i = 0; i < 2; i++) {
          for (int j = 0; j < 2; j++) {
            renderHeaderCell(header, dto.getTimetable().getArrivalStation());
          }
          for (int j = 0; j < 2; j++) {
            renderHeaderCell(header, dto.getTimetable().getDepartureStation());
          }
        }
        renderHeaderCell(header, dto.getTimetable().getArrivalStation());
        round++;
      }else{
        headerWrap.add(new AttributeModifier("class", "headerTableInvisible"));
        AbstractItem headerItem = new AbstractItem(header.newChildId());
        headerItem.add(new Label("content", ""));
        header.add(headerItem);
      }
      if(round == 0 && !isEmpty)
        headerWrap.add(new AttributeModifier("class", "headerTableRelative"));
      headerWrap.add(header);
      if(firstHeader == null){
        headerWrap.add(new AttributeModifier("class", "headerTableFixed"));
        firstHeader = headerWrap;
      }
      item.add(headerWrap);
    }

    private void renderHeaderCell(RepeatingView header, StationDTO stat) {
      AbstractItem headerItem;
      headerItem = new AbstractItem(header.newChildId());
      headerItem.add(new Label("content", stat.getCity().getCityName()));
      header.add(headerItem);
    }

    @Override
    protected void populateItem(ListItem<List<BusConnectionDTO>> item) {
      Tuple<StationDTO, StationDTO> thisSt = new Tuple<StationDTO, StationDTO>(
          item.getModelObject().get(0).getTimetable().getDepartureStation(), 
          item.getModelObject().get(0).getTimetable().getArrivalStation());
      
      /** Pokud se pocatecni dvojice neshoduje s dvojici v naposledy generovanem panelu, znaci to zmenu
       * zmena muze byt prvni takova, v tom pripade se jen nastavi isEmpty na true 
       * pokud uz se program nachazi v "empty zone" a je treba se vratit isEmpty=false a vygenerovat novou hlavicku (round=0) 
       * Navic potrebuju zmenit plovouci hlavicku na fixni, protoze jich na strance bude vice. (fixed -> relative)  */
      if(!thisSt.equals(recentSt)){
        if(isEmpty) {
          isEmpty = false;
          round = 0;
          firstHeader.add(new AttributeModifier("class", "headerTableRelative"));
        }
        else isEmpty = true;
      }
      final WaysPanel panel = new WaysPanel("lines", item.getModel(), person, listContainer, isEmpty);
       
      recentSt = thisSt;
      panels.add(panel);
      item.add(panel);
      afterPopulateItem(item);
    }
    
    @Override
    protected void onAfterRender() {
      super.onAfterRender();
      isEmpty = false;
      firstHeader = null;
    }
  }

}
