package cz.sa.shiftPlanner.web.panels.personDetail;

import java.text.DateFormatSymbols;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.SortedSet;
import java.util.TreeSet;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.ajax.markup.html.form.AjaxButton;
import org.apache.wicket.authroles.authorization.strategies.role.metadata.MetaDataRoleAuthorizationStrategy;
import org.apache.wicket.extensions.yui.calendar.DateTimeField;
import org.apache.wicket.markup.head.IHeaderResponse;
import org.apache.wicket.markup.head.JavaScriptHeaderItem;
import org.apache.wicket.markup.html.IHeaderContributor;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Button;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.list.AbstractItem;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.request.resource.JavaScriptResourceReference;
import org.apache.wicket.spring.injection.annot.SpringBean;
import org.joda.time.DateTime;

import com.google.common.collect.Lists;

import cz.sa.shiftPlanner.dto.PersonDTO;
import cz.sa.shiftPlanner.dto.TimeTrialDTO;
import cz.sa.shiftPlanner.services.PersonService;
import cz.sa.shiftPlanner.services.PersonUtilsService;
import cz.sa.shiftPlanner.utils.DateFormatter;
import cz.sa.shiftPlanner.utils.PlannerRoles;
import cz.sa.shiftPlanner.utils.TimeRange;
import cz.sa.shiftPlanner.web.MO.RangeMO;
import cz.sa.shiftPlanner.web.auth.PlannerSession;
import cz.sa.shiftPlanner.web.panels.MultiDatePicker;
import cz.sa.shiftPlanner.web.panels.PAbstractPanel;
import cz.sa.shiftPlanner.web.utils.RangeMOComparator;

/**
 * @author ToSk
 */

public class TimeTrialPanel extends Panel{

	@SpringBean(name = "personUtilsService")
	PersonUtilsService personUtilsService;
	
	private PersonDTO person;
	private final WebMarkupContainer viewContainer;
	private LoadableDetachableModel<List<TimeTrialDTO>> ttModel;
	private Form form;
	private IModel<String> monthModel = new Model<String>();
	private SortedSet<RangeMO> dates = new TreeSet<RangeMO>(new RangeMOComparator());
	
	public TimeTrialPanel(String id, final PersonDTO person) {
		super(id);
		this.person = person;
		viewContainer = new WebMarkupContainer("container");
		viewContainer.setOutputMarkupId(true);
		add(viewContainer);
		add(new Label("message", "Casovky"));
		
		Form datesForm = new Form("datesForm");
		final MultiDatePicker picker = new MultiDatePicker("datePicker");
		Button saveAll = new Button("saveAll") {     
      public void onSubmit() {
        dates = picker.getDates();
        if(dates.size() > 0){
          List<TimeRange> ranges = new ArrayList<TimeRange>();
          for (RangeMO r : dates) {
            ranges.add(new TimeRange(new DateTime(r.getFrom().getObject()), new DateTime(r.getTo().getObject())));
          }
          personUtilsService.createTrials(ranges, person);
          picker.clearSet();
        }
      }
    };
    datesForm.add(saveAll, picker);
    add(datesForm);
		
		//create instance form
		viewContainer.add(form = new TtForm("ttForm", new TimeTrialDTO()));
		 MetaDataRoleAuthorizationStrategy.authorize(form, RENDER, PlannerRoles.STEWARD );
		
		 Form monthForm = new Form("monthForm");
		 Locale locale = new Locale("cs", "CZ");
		 final List<String> czMonths = Lists.newArrayList(new DateFormatSymbols(locale).getMonths());
		 monthModel.setObject(new DateTime().monthOfYear().getAsText(locale));
		 DropDownChoice choice = new DropDownChoice("months", monthModel, Lists.newArrayList(czMonths));
		 monthForm.add(choice);
		 AjaxButton refresh = new AjaxButton("setMonth") {     
			    protected void onSubmit(AjaxRequestTarget target, Form form) {
			      target.appendJavaScript("hideEditTool();");
			      target.add(viewContainer);
			    }
			};
			monthForm.add(refresh);
			viewContainer.add(monthForm);
		
		//nacitani casovek
		ttModel = new LoadableDetachableModel<List<TimeTrialDTO>>() {
			
			@Override
			protected List<TimeTrialDTO> load() {
				return personUtilsService.findAllByPersonId(person.getId(), czMonths.indexOf(monthModel.getObject())+1, PlannerSession.get().getSelectedYear());
			}
		};
		renderViewTable();
	}

	private class TtForm extends Form<TimeTrialDTO>{

		@SpringBean(name = "personService")
		PersonService personService;
		
		private IModel<Date> dateModelFrom;
		private IModel<Date> dateModelTo;
		private TextArea<String> textArea;
		
		public TtForm(String id, TimeTrialDTO ex) {
			super(id, new CompoundPropertyModel<TimeTrialDTO>(ex));
			
			add(new AjaxButton("ajaxButton") {
				@Override
				protected void onSubmit(AjaxRequestTarget target, Form<?> form) {
					//naplneni excuse daty ktera se nenaplni automaticky z compoundPropertyModelu
					TimeTrialDTO tt = TtForm.this.getModelObject();
					tt.setPerson(person);
					tt.setFrom_(new DateTime(dateModelFrom.getObject()));
					tt.setTo_(new DateTime(dateModelTo.getObject()));
					if(tt.getId() == null)
						personUtilsService.createTrial(tt);
					else
						personUtilsService.updateTrial(tt);
					//reset formulare
					TtForm.this.setModelObject(new TimeTrialDTO());
					target.appendJavaScript("hideEditTool();");
					target.add(viewContainer);
				}
			});
			
			add(new Label("person_name", new PropertyModel<String>(person, "name")));
			add(new Label("person_surname", new PropertyModel<String>(person, "surname")));
			
			
			//nastaveni modelu tak aby vytvarel novou instanci date nebo nacital existujici pri editovani
			dateModelFrom = new LoadableDetachableModel<Date>() {
				@Override
				protected Date load() {
					if(getModelObject().getFrom_() == null){
						return DateFormatter.toBeginOfDay(new DateTime()).toDate();
					}
					return getModelObject().getFrom_().toDate();
				}
			};
			
			DateTimeField calendar = new DateTimeField("calendar",dateModelFrom);
			calendar.setRequired(true);
	    	add(calendar);
	    	
	    	dateModelTo = new LoadableDetachableModel<Date>() {
				@Override
				protected Date load() {
					if(getModelObject().getFrom_() == null){
						return DateFormatter.toEndOfDay(new DateTime()).toDate();
					}
					return getModelObject().getTo_().toDate();
				}
			};
	    	
			DateTimeField calendar2 = new DateTimeField("calendar2",dateModelTo);
	   	calendar2.setRequired(true);
	   	add(calendar2);
	    	
	    	add(textArea = new TextArea<String>("note"));
		}
		
		
	}
	
	private void renderViewTable() {
		
		
		Component header = new AbstractItem("header");
		ListView<TimeTrialDTO> repeatingView = new ListView<TimeTrialDTO>("repeating", ttModel){
			
			@Override
			protected void populateItem(final ListItem<TimeTrialDTO> item) {
				
				item.add(new Label("dateFrom", String.valueOf(DateFormatter.toCompletTimestamp(item.getModelObject().getFrom_()))));
				item.add(new Label("dateTo", String.valueOf(DateFormatter.toCompletTimestamp(item.getModelObject().getTo_()))));
				item.add(new Label("text", item.getModelObject().getNote() == null ? "" : item.getModelObject().getNote()));
				AjaxLink delete = new AjaxLink("crossLink") {
	                @Override
	                public void onClick(AjaxRequestTarget target) {
	                    personUtilsService.remove(item.getModelObject());
	                    ttModel.detach();
	                    target.appendJavaScript("hideEditTool();");
	                    target.add(viewContainer);
	                }
	            };
	            
	            AjaxLink editLink = new AjaxLink<Void>("editLink") {
	            	@Override
	            	public void onClick(AjaxRequestTarget target)
	            	{
	            		form.setModelObject(item.getModelObject());
	            		target.appendJavaScript("showEditTool();");
	            		target.add(viewContainer);
	            	}
	            };
	            MetaDataRoleAuthorizationStrategy.authorize(editLink, RENDER, PlannerRoles.STEWARD);
	            MetaDataRoleAuthorizationStrategy.authorize(delete, RENDER, PlannerRoles.STEWARD);
	            item.add(delete, editLink);
				
				if(item.getIndex()%2==1) item.add(new AttributeModifier("class", "even"));
				else item.add(new AttributeModifier("class", "odd"));
			}
		};
		viewContainer.add(header, repeatingView);

		
	}
	
	@Override
	protected void detachModel() {
		super.detachModel();
		ttModel.detach();
	}
	
}


