package cz.sa.shiftPlanner.web.panels.personDetail;

import java.text.DateFormatSymbols;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.ajax.markup.html.form.AjaxButton;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.spring.injection.annot.SpringBean;
import org.joda.time.DateTime;

import com.google.common.collect.Lists;

import cz.sa.shiftPlanner.dto.PersonDTO;
import cz.sa.shiftPlanner.dto.WayDTO;
import cz.sa.shiftPlanner.services.PersonService;
import cz.sa.shiftPlanner.services.PersonUtilsService;
import cz.sa.shiftPlanner.services.WayService;

/**
 * @author ToSk
 */

public class WayContainer extends Panel {

  @SpringBean(name = "wayService")
  WayService wayService;

  @SpringBean(name = "personService")
  PersonService personService;

  @SpringBean(name = "personUtilsService")
  PersonUtilsService personUtilsService;

  private IModel<String> monthModel = new Model<String>();
  private List<WaysViewPanel> panels = new ArrayList<WaysViewPanel>();
  private WebMarkupContainer listContainer;

  public WayContainer(String id, final PersonDTO person) {
    super(id);

    final Form form = new Form("form");

    listContainer = new WebMarkupContainer("container");
    listContainer.setOutputMarkupId(true);

    Locale locale = new Locale("cs", "CZ");
    final List<String> czMonths = Lists.newArrayList(new DateFormatSymbols(locale).getMonths());
    monthModel.setObject(new DateTime().monthOfYear().getAsText(locale));
    DropDownChoice choice = new DropDownChoice("months", monthModel, Lists.newArrayList(czMonths));
    choice.add(new AjaxFormComponentUpdatingBehavior("onchange") {

      @Override
      protected void onUpdate(AjaxRequestTarget target) {
        target.add(listContainer);
      }
    });
    form.add(choice);

    //pocet odpracovanych hodin a mesicni limit
    final IModel<Integer> sum = new LoadableDetachableModel<Integer>() {
      @Override
      protected Integer load() {
        return personUtilsService.sumMonthHours(person, czMonths.indexOf(monthModel.getObject()) + 1, new DateTime().getYear()) / 60;
      }
    };

    final IModel<Long> limit = new LoadableDetachableModel<Long>() {
      @Override
      protected Long load() {
        return personUtilsService.limitMonthHours(person, czMonths.indexOf(monthModel.getObject()) + 1,
            new DateTime().getYear()) / 60;
      }
    };
    Label workHours = new Label("workHours", sum);
    listContainer.add(workHours, new Label("limitHours", limit));

    final IModel<List<List<WayDTO>>> model = new LoadableDetachableModel<List<List<WayDTO>>>() {
      @Override
      protected List<List<WayDTO>> load() {
        return wayService.findByMonth(czMonths.indexOf(monthModel.getObject()) + 1, person);
      }
    };

    LineViews views = new LineViews("times", model);
    listContainer.add(views);
    form.add(listContainer);
    add(form);
  }

  private class LineViews extends ListView<List<WayDTO>> {

    private IModel<PersonDTO> person;

    public LineViews(String id, IModel<List<List<WayDTO>>> model) {
      super(id, model);
    }

    @Override
    protected void populateItem(ListItem<List<WayDTO>> item) {
      final WaysViewPanel panel = new WaysViewPanel("lines", item.getModel());
      panels.add(panel);
      item.add(panel);
    }
  }
}
