package cz.sa.shiftPlanner.web.panels.personDetail;

import java.util.List;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.authroles.authorization.strategies.role.metadata.MetaDataRoleAuthorizationStrategy;
import org.apache.wicket.extensions.ajax.markup.html.AjaxEditableMultiLineLabel;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.AbstractItem;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.RepeatingView;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.ResourceModel;
import org.apache.wicket.model.StringResourceModel;
import org.joda.time.DateTime;

import cz.sa.shiftPlanner.dto.BusConnectionDTO;
import cz.sa.shiftPlanner.dto.WayDTO;
import cz.sa.shiftPlanner.utils.PlannerRoles;

/**
 * @author ToSk
 */

public class WaysViewPanel extends Panel{

	private  IModel<List<WayDTO>> lines;
	
	public WaysViewPanel(String id,  IModel<List<WayDTO>> lines) {
		super(id, lines);
		this.lines = lines;
		
		Label dayNote = new Label("dayNote", dateFormatToDay(lines.getObject().get(0).getDepartureTime()));
		Label absenceNote = new Label("absenceNote","");
		WayDTO w;
		if((w = hasAnyExcuse()) != null){
		  absenceNote = new Label("absenceNote", new ResourceModel("absence.reason."+w.getExcuse().getReason()));
		}
		add(dayNote, absenceNote);
		drawComponent();
	}


  private void drawComponent(){
	
	  Label busNumber = new Label("busNumber", "BUS " + new PropertyModel<Integer>(lines.getObject().get(0).getConnection().getBusTurn(), "coupleNumber").getObject());
	  add(busNumber);
	  
	  Label noteBefore = new Label("noteBefore", new PropertyModel(lines.getObject().get(0).getConnection().getBusTurn(), "noteBefore"));
	  add(noteBefore);
	  
	  Label noteAfter = new Label("noteAfter", new PropertyModel(lines.getObject().get(lines.getObject().size() - 1).getConnection().getBusTurn(), "noteAfter"));
	  add(noteAfter);
		
	  RepeatingView rows = new RepeatingView("rows");
		add(rows);
		
    
		for(int i=0; i< 3; i++){
			AbstractItem rowsItem = new AbstractItem(rows.newChildId());
			
			RepeatingView cols = new RepeatingView("cols");
			rowsItem.add(cols);
			
			switch(i){
				
				case 0 : {
					
					
					for (WayDTO busT : lines.getObject()) {
						
						AbstractItem colsItem = new AbstractItem(cols.newChildId());
						
						colsItem.add(new Label("label", "odj: "+ busT.getConnection().getTimetable().getDepartureStation().getCity().getCityName()));
						if(!busT.getIsActive())
						  colsItem.add(new AttributeModifier("style", "background-color: lightgoldenrodyellow"));
						cols.add(colsItem);
							
						colsItem = new AbstractItem(cols.newChildId());
						colsItem.add(new Label("label", "prij: "+ busT.getConnection().getTimetable().getArrivalStation().getCity().getCityName()));
						if(!busT.getIsActive())
              colsItem.add(new AttributeModifier("style", "background-color: lightgoldenrodyellow"));
							
						cols.add(colsItem);
					}
					rows.add(rowsItem);
					break; 
				}	
				case 1:{
					
					for (WayDTO busT : lines.getObject()) {
						
						AbstractItem colsItem = new AbstractItem(cols.newChildId());
						
						colsItem.add(new Label("label", dateFormatToTime(busT.getConnection().getDepartureTime())));
						if(!busT.getIsActive())
              colsItem.add(new AttributeModifier("style", "background-color: lightgoldenrodyellow"));
						cols.add(colsItem);
						
						colsItem = new AbstractItem(cols.newChildId());
						colsItem.add(new Label("label", dateFormatToTime(busT.getConnection().getArrivalTime())));
						if(!busT.getIsActive())
              colsItem.add(new AttributeModifier("style", "background-color: lightgoldenrodyellow"));
						cols.add(colsItem);
					}
					rows.add(rowsItem);
					break;
				}
			}
		}
	}
	
	private String dateFormatToTime(DateTime time){
		String sTime;
		sTime= String.format("%02d",time.getHourOfDay()) +" : "+ String.format("%02d",time.getMinuteOfHour());
		return sTime;
	}
	
	private String dateFormatToDay(DateTime time){
    	String sTime;
    	sTime= time.getDayOfMonth() + "." + time.getMonthOfYear() + "." + time.getYear();
    	return sTime;
    }
	
	private WayDTO hasAnyExcuse() {
	  for (WayDTO w : lines.getObject()) {
      if(!w.getIsActive()){
        return w;
      }
    }
	  return null;
	}
}


