package cz.sa.shiftPlanner.web.wizard;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.wicket.Component;
import org.apache.wicket.extensions.ajax.markup.html.tabs.AjaxTabbedPanel;
import org.apache.wicket.extensions.markup.html.tabs.AbstractTab;
import org.apache.wicket.extensions.markup.html.tabs.ITab;
import org.apache.wicket.extensions.wizard.IWizardStep;
import org.apache.wicket.extensions.wizard.Wizard;
import org.apache.wicket.extensions.wizard.WizardModel;
import org.apache.wicket.extensions.wizard.WizardStep;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.spring.injection.annot.SpringBean;
import org.joda.time.DateTime;

import com.google.common.collect.Lists;

import cz.sa.shiftPlanner.dto.BusConnectionDTO;
import cz.sa.shiftPlanner.dto.LineGroupDTO;
import cz.sa.shiftPlanner.services.BusConnectionService;
import cz.sa.shiftPlanner.services.LineGroupService;
import cz.sa.shiftPlanner.services.PersonService;
import cz.sa.shiftPlanner.utils.DateFormatter;
import cz.sa.shiftPlanner.utils.TimeRange;
import cz.sa.shiftPlanner.web.MenuPage;
import cz.sa.shiftPlanner.web.MO.WayMO;
import cz.sa.shiftPlanner.web.panels.CalendarPanel;
import cz.sa.shiftPlanner.web.panels.CentralWaysPanel;

public class WayWizard extends Wizard {

	@SpringBean(name = "lineGroupService")
	LineGroupService lineService;

	private IModel<Date> dateModelFrom = new Model<Date>();
	private IModel<Date> dateModelTo = new Model<Date>();
	private PageParameters parameters;
	private LineGroupDTO lineGroup;

	public WayWizard(String id, PageParameters parameters) {
		super(id);

		this.parameters = parameters;
		this.lineGroup = lineService.get(parameters.get("id").toLong());

		// Dynamic
		WizardModel model = new WizardModel();
		model.add(new CalendarPicker(dateModelFrom, dateModelTo));
		model.add(new Ways(dateModelFrom, dateModelTo, this.parameters));

		init(model);
	}

	private final class CalendarPicker extends WizardStep {
		public CalendarPicker(final IModel<Date> dateModelFrom, final IModel<Date> dateModelTo) {
			super("", "Jizdy autobusu pro linku: "
					+ lineGroup.getName());

			dateModelFrom.setObject(new Date());
			CalendarPanel calendar = new CalendarPanel("calendar", dateModelFrom);
			add(calendar);

			dateModelTo.setObject(new DateTime().plusDays(7).toDate());
			CalendarPanel calendar2 = new CalendarPanel("calendar2", dateModelTo);
			add(calendar2);
		}

	}

	@Override
	public void onActiveStepChanged(IWizardStep newStep) {
		super.onActiveStepChanged(newStep);
	}

	private final class Ways extends WizardStep {

		@SpringBean(name = "busConnectionService")
		BusConnectionService busConnectionService;

		@SpringBean(name = "personService")
		PersonService personService;
		
		private IModel<Date> dateModelFrom;
		private IModel<Date> dateModelTo;
		private PageParameters parameters;
		private List<IModel<List<List<BusConnectionDTO>>>> models = new ArrayList<IModel<List<List<BusConnectionDTO>>>>();
		

		public Ways(final IModel<Date> dateModelFrom, final IModel<Date> dateModelTo, final PageParameters parameters) {
			super("", "Jizdy autobusu pro linku: "
					+ lineGroup.getName());

			this.dateModelFrom = dateModelFrom;
			this.dateModelTo = dateModelTo;
			this.parameters = parameters;
		}

		@Override
		protected void onBeforeRender() {
			super.onBeforeRender();

			createPanels(this.dateModelFrom, this.dateModelTo, this.parameters);
		}

		@Override
		protected void detachModel() {
			super.detachModel();
			for (IModel<List<List<BusConnectionDTO>>> model : models) {
				model.detach();
			}
		}
		

		private void createPanels(final IModel<Date> dateModelFrom, final IModel<Date> dateModelTo,
				final PageParameters parameters) {

			List<ITab> tabs = new ArrayList<ITab>();
			DateTime from = new DateTime(dateModelFrom.getObject());
			DateTime to = new DateTime(dateModelTo.getObject());

			
			while (from.compareTo(to) <= 0) {
				final DateTime tmpFrom = from;
				final IModel<List<List<BusConnectionDTO>>> connModel = new LoadableDetachableModel<List<List<BusConnectionDTO>>>() {
					@Override
					protected List<List<BusConnectionDTO>> load() {
						return Lists.newArrayList(busConnectionService.findConnectionByTimeRange(
						    new TimeRange(DateFormatter.toBeginOfDay(tmpFrom), DateFormatter.toBeginOfDay(tmpFrom.plusDays(1))),
						    lineGroup));
					}
					@Override
					public void detach() {
					}
				};
				
				IModel<List<BusConnectionDTO>> suppModel = new LoadableDetachableModel<List<BusConnectionDTO>>() {
					@Override
					protected List<BusConnectionDTO> load() {
						return Lists.newArrayList(busConnectionService.findSupportsByTimeRange(
								tmpFrom, parameters.get("id").toLongObject()));
					}
				};
				final WayMO wayMO = new WayMO(connModel, suppModel, tmpFrom);
				
				 tabs.add(new AbstractTab(new Model<String>(dateFormatToDate(from))) {
					public Panel getPanel(String panelId) {
						return new CentralWaysPanel(panelId, wayMO, parameters);
					}
				});

				models.add(connModel);
				from = from.plusDays(1);
			}

			addOrReplace(new AjaxTabbedPanel("tabs", tabs));

		}

		private String dateFormatToDate(DateTime time) {
			String sTime;
			sTime = String.valueOf(time.dayOfWeek().getAsText()) + " - " + time.getDayOfMonth()
					+ "." + time.getMonthOfYear();
			return sTime;
		}

	}

	@Override
	public void onCancel() {
		setResponsePage(MenuPage.class);
	}

	@Override
	public void onFinish() {

		setResponsePage(WizardPage.class, parameters);
	}

	@Override
	protected Component newFeedbackPanel(String id) {
		Component fp = super.newFeedbackPanel(id);
		fp.setVisible(false);
		return fp;
	}
}
